@@ -7,7 +7,7 @@ PKG_CONFIG_PATH="/usr/local/lib/pkgconfig" \
 LDFLAGS="-L/usr/local/lib" \
 CFLAGS="-O2 -I/usr/local/include -static -no-pie" \
 CXXFLAGS="-O2 -I/usr/local/include -static -no-pie" \
-meson /root/tmp/builddir \
+meson setup /root/tmp/builddir \
 -D88-colors=true \
 -D256-colors=true \
 -Dbacktrace=false \
@@ -16,11 +16,11 @@ meson /root/tmp/builddir \
 -Dbzlib=true \
 -Dcgi=true \
 -Dcss=true \
--Dcombining=false \
 -Ddgi=true \
 -Dexmode=true \
 -Dfastmem=true \
 -Dfsp=false \
+-Dfsp2=true \
 -Dgemini=true \
 -Dgettext=false \
 -Dgnutls=false \
@@ -56,10 +56,10 @@ meson /root/tmp/builddir \
 -Dx=false \
 -Dxbel=true \
 -Dzlib=true \
--Dzstd=true
+-Dzstd=true || exit 1
 
-meson compile -C /root/tmp/builddir
+meson compile -C /root/tmp/builddir || exit 2
 
-strip /root/tmp/builddir/src/elinks
+strip /root/tmp/builddir/src/elinks || exit 3
 
-upx /root/tmp/builddir/src/elinks
+upx /root/tmp/builddir/src/elinks || exit 4
