@@ -4,7 +4,7 @@ rm -rf /root/tmp/builddir_js
 
 CFLAGS="-O2 -static -no-pie" \
 CXXFLAGS="-O2 -static -no-pie" \
-meson /root/tmp/builddir_js \
+meson setup /root/tmp/builddir_js \
 -D88-colors=true \
 -D256-colors=true \
 -Dbacktrace=false \
@@ -13,11 +13,11 @@ meson /root/tmp/builddir_js \
 -Dbzlib=true \
 -Dcgi=true \
 -Dcss=true \
--Dcombining=false \
 -Ddgi=true \
 -Dexmode=true \
 -Dfastmem=true \
 -Dfsp=false \
+-Dfsp2=true \
 -Dgemini=true \
 -Dgettext=false \
 -Dgnutls=false \
@@ -26,6 +26,8 @@ meson /root/tmp/builddir_js \
 -Dguile=false \
 -Didn=true \
 -Dipv6=true \
+-Dlibcss=true \
+-Dlibcurl=true \
 -Dlibev=false \
 -Dlibevent=true \
 -Dluapkg='luajit' \
@@ -51,6 +53,10 @@ meson /root/tmp/builddir_js \
 -Dx=false \
 -Dxbel=true \
 -Dzlib=true \
--Dzstd=true
+-Dzstd=true || exit 1
 
-meson compile -j $(($(nproc) - 1)) -C /root/tmp/builddir_js
+meson compile -j $(($(nproc) - 1)) -C /root/tmp/builddir_js || exit 2
+
+strip /root/tmp/builddir_js/src/elinks || exit 3
+
+upx /root/tmp/builddir_js/src/elinks || exit 4
