@@ -166,8 +166,10 @@ lookup_cmd(struct option *o, char ***argv, int *argc)
 {
 	struct sockaddr_storage *addrs = NULL;
 	int addrno, i;
-	char *idname, *idname2;
-	int allocated = 0;
+	char *idname;
+#ifdef CONFIG_IDN2
+	char *idname2 = NULL;
+#endif
 
 	if (!*argc) return gettext("Parameter expected");
 	if (*argc > 1) return gettext("Too many parameters");
@@ -182,7 +184,6 @@ lookup_cmd(struct option *o, char ***argv, int *argc)
 
 		if (code == IDN2_OK) {
 			idname = idname2;
-			allocated = 1;
 		}
 	}
 #endif
@@ -193,9 +194,12 @@ lookup_cmd(struct option *o, char ***argv, int *argc)
 #else
 		usrerror(gettext("Host not found"));
 #endif
-		if (allocated) {
+
+#ifdef CONFIG_IDN2
+		if (idname2) {
 			free(idname2);
 		}
+#endif
 		return "";
 	}
 
@@ -224,9 +228,11 @@ lookup_cmd(struct option *o, char ***argv, int *argc)
 
 	fflush(stdout);
 
-	if (allocated) {
+#ifdef CONFIG_IDN2
+	if (idname2) {
 		free(idname2);
 	}
+#endif
 
 	return "";
 }
@@ -834,6 +840,10 @@ union option_info cmdline_options_info[] = {
 
 	INIT_OPT_COMMAND("", NULL, "confdir", OPT_HIDDEN, redir_cmd, NULL),
 
+	INIT_OPT_CMDALIAS("", N_("Set mode of displaying colors in documents"),
+		"colors", OPT_ALIAS, "document.colors.use_document_colors",
+		N_("Possible values 0, 1 or 2.")),
+
 	INIT_OPT_STRING("", N_("Name of directory with configuration file"),
 		"config-dir", OPT_ZERO, "",
 		N_("Path of the directory ELinks will read and write its "
@@ -869,6 +879,12 @@ union option_info cmdline_options_info[] = {
 		"be ignored. It forces use of default keybindings and will "
 		"reset user-defined ones on save.")),
 
+	INIT_OPT_BOOL("", N_("Delete timer queue"),
+		"delete-timer-queue", OPT_ZERO, 0,
+		N_("Whether to delete timer queue under Windows. "
+		"When QuickJS is enabled on Win32, you can enable this option to disable "
+		"js execution timeouts.")),
+
 	INIT_OPT_BOOL("", N_("Print formatted versions of given URLs to stdout"),
 		"dump", OPT_ZERO, 0,
 		N_("Print formatted plain-text versions of given URLs to "
@@ -1009,6 +1025,10 @@ union option_info cmdline_options_info[] = {
 		N_("When enabled, terminfo ncurses functions will be used "
 		"instead of hardcoded sequences.")),
 
+	INIT_OPT_BOOL("", N_("Test js"),
+		"test", OPT_ZERO, 0,
+		N_("When enabled, helps to test js.")),
+
 	INIT_OPT_BOOL("", N_("Touch files in ~/.config/elinks when running with -no-connect/-session-ring"),
 		"touch-files", OPT_ZERO, 0,
 		N_("When enabled, runtime state files (bookmarks, history, "
