@@ -518,7 +518,7 @@ init_option_listbox_item(struct option *option)
 struct option *
 add_opt(struct option *tree, const char *path, const char *capt,
 	const char *name, option_flags_T flags, enum option_type type,
-	long min, long max, longptr_T value, const char *desc)
+	long min, long max, intptr_t value, const char *desc)
 {
 	struct option *option = (struct option *)mem_calloc(1, sizeof(*option));
 
