@@ -63,7 +63,7 @@
 #include "document/renderer.h"
 
 #if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
-#include "ecmascript/ecmascript-c.h"
+#include "js/ecmascript-c.h"
 #endif
 
 #ifdef CONFIG_LIBDOM
@@ -213,17 +213,12 @@ init_document(struct cache_entry *cached, struct document_options *options)
 
 #ifdef CONFIG_ECMASCRIPT
 	init_list(document->onload_snippets);
-	init_list(document->timeouts);
 #endif
 
 #ifdef CONFIG_LIBSIXEL
 	init_list(document->images);
 #endif
 
-#ifdef CONFIG_COMBINE
-	document->comb_x = -1;
-	document->comb_y = -1;
-#endif
 	object_nolock(document, "document");
 	object_lock(document);
 
@@ -357,8 +352,6 @@ reset_document(struct document *document)
 	free_ecmascript_string_list(&document->onload_snippets);
 	free_uri_list(&document->ecmascript_imports);
 	mem_free_set(&document->body_onkeypress, NULL);
-///	kill_timer(&document->timeout);
-///	free_document(document->dom);
 #endif
 
 	free_list(document->tags);
@@ -368,10 +361,6 @@ reset_document(struct document *document)
 	mem_free_set(&document->slines1, NULL);
 	mem_free_set(&document->slines2, NULL);
 	mem_free_set(&document->search_points, NULL);
-
-#ifdef CONFIG_COMBINE
-	discard_comb_x_y(document);
-#endif
 }
 
 void
@@ -432,8 +421,6 @@ done_document(struct document *document)
 #if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
 	free_ecmascript_string_list(&document->onload_snippets);
 	free_uri_list(&document->ecmascript_imports);
-	kill_ecmascript_timeouts(document);
-	free_list(document->timeouts);
 	mem_free_if(document->body_onkeypress);
 #endif
 
@@ -442,15 +429,11 @@ done_document(struct document *document)
 	free_document(document->dom);
 
 	if (document->element_map) {
-		void *mapa = document->element_map;
-
-		delete_map(&mapa);
+		delete_map(document->element_map);
 	}
 
 	if (document->element_map_rev) {
-		void *mapa = document->element_map_rev;
-
-		delete_map_rev(&mapa);
+		delete_map(document->element_map_rev);
 	}
 #endif
 	free_list(document->tags);
@@ -471,11 +454,9 @@ release_document(struct document *document)
 	assert(document);
 	if_assert_failed return;
 
-	if (document->refresh) kill_document_refresh(document->refresh);
-#if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
-	kill_ecmascript_timeouts(document);
-	free_list(document->timeouts);
-#endif
+	if (document->refresh) {
+		kill_document_refresh(document->refresh);
+	}
 	object_unlock(document);
 	move_document_to_top_of_format_cache(document);
 }
@@ -569,8 +550,8 @@ get_cached_document(struct cache_entry *cached, struct document_options *options
 		    || compare_opt(&document->options, options))
 			continue;
 
-		if (options->no_cache
-		    || cached->cache_id != document->cache_id
+		if (
+		    cached->cache_id != document->cache_id
 		    || !check_document_css_magic(document)) {
 			if (!is_object_used(document)) {
 				add_to_document_list(&to_remove, document);
@@ -730,5 +711,6 @@ struct module document_module = struct_module(
 	/* submodules: */	NULL,
 	/* data: */		NULL,
 	/* init: */		init_documents,
-	/* done: */		done_documents
+	/* done: */		done_documents,
+	/* getname: */	NULL
 );
