@@ -21,6 +21,7 @@ struct ecmascript_timeout;
 struct el_form_control;
 struct frame_desc;
 struct frameset_desc;
+struct hash;
 struct image;
 struct module;
 struct screen_char;
@@ -230,8 +231,6 @@ struct document {
 	 * dependencies between the various entries so nothing gets removed
 	 * unneeded. */
 	struct uri_list ecmascript_imports;
-	/** used by setTimeout */
-	LIST_OF(struct ecmascript_timeout) timeouts;
 	int ecmascript_counter;
 	char *body_onkeypress;
 #endif
@@ -241,6 +240,7 @@ struct document {
 	void *element_map_rev;
 	struct string text;
 	void *forms_nodeset;
+	struct hash *hh;
 #endif
 
 #ifdef CONFIG_CSS
@@ -285,14 +285,6 @@ struct document {
 #ifdef CONFIG_UTF8
 	char buf[7];
 	unsigned char buf_length;
-#endif
-#ifdef CONFIG_COMBINE
-	/* base char + 5 combining chars = 6 */
-	unicode_val_T combi[UCS_MAX_LENGTH_COMBINED];
-	/* the number of combining characters. The base char is not counted. */
-	unsigned int combi_length;
-	/* Positions of the last base character.*/
-	int comb_x, comb_y;
 #endif
 	unsigned int cache_id; /**< Used to check cache entries. */
 
