@@ -322,6 +322,7 @@ not_processed:
 
 		char *import_url;
 		struct uri *uri;
+		struct string tmp;
 
 		if (!get_opt_bool("ecmascript.enable", NULL)) {
 			mem_free(src);
@@ -340,10 +341,22 @@ not_processed:
 		html_context->special_f(html_context, SP_SCRIPT, uri);
 		done_uri(uri);
 
-		/* Create URL reference onload snippet. */
-		insert_in_string(&import_url, 0, "^", 1);
-		add_to_ecmascript_string_list(&html_context->part->document->onload_snippets,
-		                   import_url, -1, html_top->name - html_context->part->document->text.source);
+		if (init_string(&tmp)) {
+			char *integrity = get_attr_val(a, "integrity", html_context->doc_cp);
+
+			add_char_to_string(&tmp, '^');
+
+			if (integrity) {
+				add_to_string(&tmp, integrity);
+				mem_free(integrity);
+			}
+			add_char_to_string(&tmp, '\0');
+			add_to_string(&tmp, import_url);
+			/* Create URL reference onload snippet. */
+			add_to_ecmascript_string_list(&html_context->part->document->onload_snippets,
+				tmp.source, tmp.length, html_top->name - html_context->part->document->text.source);
+			done_string(&tmp);
+		}
 
 imported:
 		/* Retreat. Do not permit nested scripts, tho'. */
@@ -374,6 +387,9 @@ imported:
 		}
 	}
 
+	if (eof - html >= (sizeof("<![CDATA[") - 1) && !strncmp(html, "<![CDATA[", (sizeof("<![CDATA[") - 1))) {
+		html += (sizeof("<![CDATA[") - 1);
+	}
 	*end = html;
 
 	/* Now look ahead for the script end. The <script> contents is raw
@@ -402,6 +418,7 @@ imported:
 			 * extreme ;-). */
 		}
 
+
 		if (**end != '<')
 			continue;
 		/* We want to land before the closing element, that's why we
@@ -1175,3 +1192,10 @@ html_noscript(struct html_context *html_context, char *a,
 		html_skip(html_context, a);
 #endif
 }
+
+void
+html_template(struct html_context *html_context, char *a,
+              char *xxx3, char *xxx4, char **xxx5)
+{
+	html_skip(html_context, a);
+}
