@@ -524,6 +524,7 @@ static struct element_info elements[] = {
  {"SUP",         html_superscript, NULL,                 0, ET_NESTABLE    },
  {"TABLE",       html_table,       NULL,                 2, ET_NESTABLE    },
  {"TD",          html_td,          NULL,                 0, ET_NESTABLE    },
+ {"TEMPLATE",    html_template,    NULL,                 0, ET_NON_NESTABLE},
  {"TEXTAREA",    html_textarea,    NULL,                 0, ET_NON_PAIRABLE},
  {"TH",          html_th,          NULL,                 0, ET_NESTABLE    },
  {"TITLE",       html_title,       NULL,                 0, ET_NESTABLE    },
@@ -1044,7 +1045,13 @@ start_element(struct element_info *ei,
 	ELEMENT_RENDER_PROLOGUE
 
 	/* Call the element's open handler. */
-	if (ei->open) ei->open(html_context, attr, html, eof, &html);
+	if (ei->open) {
+		if (has_attr(attr, "hidden", html_context->doc_cp)) {
+			html_skip(html_context, attr);
+		} else {
+			ei->open(html_context, attr, html, eof, &html);
+		}
+	}
 
 	/* Apply CSS styles again. */
 #ifdef CONFIG_CSS
@@ -1217,7 +1224,9 @@ process_element(char *name, int namelen, int endingtag,
 		}
 	}
 
-	if (!ei) return html;
+	if (!ei) {
+		ei = &elements[NUMBER_OF_TAGS];
+	}
 
 	if (!endingtag) {
 		return start_element(ei, name, namelen, html, eof, attr, html_context);
