@@ -78,9 +78,6 @@ static css_error set_libcss_node_data(void *pw, void *node,
 static css_error get_libcss_node_data(void *pw, void *node,
 		void **libcss_node_data);
 
-static css_error compute_font_size(void *pw, const css_hint *parent,
-		css_hint *size);
-
 static css_error named_ancestor_node(void *pw, void *node,
 		const css_qname *qname, void **ancestor);
 
@@ -100,7 +97,11 @@ static css_error node_presentational_hint(void *pw, void *node,
 	return CSS_OK;
 }
 
-
+static css_error
+resolve_url_empty(void *pw, const char *base, lwc_string *rel, lwc_string **abs)
+{
+	return CSS_OK;
+}
 
 css_error
 resolve_url(void *pw, const char *base, lwc_string *rel, lwc_string **abs)
@@ -228,7 +229,8 @@ static css_select_handler selection_handler = {
 
 
 /* Handler for libcss_node_data, stored as libdom node user data */
-static void nscss_dom_user_data_handler(dom_node_operation operation,
+static void
+nscss_dom_user_data_handler(dom_node_operation operation,
 		dom_string *key, void *data, struct dom_node *src,
 		struct dom_node *dst)
 {
@@ -391,80 +393,6 @@ css_computed_style *nscss_get_blank_style(nscss_select_ctx *ctx,
 	return composed;
 }
 
-/**
- * Font size computation callback for libcss
- *
- * \param pw      Computation context
- * \param parent  Parent font size (absolute)
- * \param size    Font size to compute
- * \return CSS_OK on success
- *
- * \post \a size will be an absolute font size
- */
-css_error compute_font_size(void *pw, const css_hint *parent, css_hint *size)
-{
-#if 0
-	static css_hint_length sizes[] = {
-		{ FLTTOFIX(6.75), CSS_UNIT_PT },
-		{ FLTTOFIX(7.50), CSS_UNIT_PT },
-		{ FLTTOFIX(9.75), CSS_UNIT_PT },
-		{ FLTTOFIX(12.0), CSS_UNIT_PT },
-		{ FLTTOFIX(13.5), CSS_UNIT_PT },
-		{ FLTTOFIX(18.0), CSS_UNIT_PT },
-		{ FLTTOFIX(24.0), CSS_UNIT_PT }
-	};
-	const css_hint_length *parent_size;
-
-	UNUSED(pw);
-
-	/* Grab parent size, defaulting to medium if none */
-	if (parent == NULL) {
-		parent_size = &sizes[CSS_FONT_SIZE_MEDIUM - 1];
-	} else {
-		assert(parent->status == CSS_FONT_SIZE_DIMENSION);
-		assert(parent->data.length.unit != CSS_UNIT_EM);
-		assert(parent->data.length.unit != CSS_UNIT_EX);
-		parent_size = &parent->data.length;
-	}
-
-	assert(size->status != CSS_FONT_SIZE_INHERIT);
-
-	if (size->status < CSS_FONT_SIZE_LARGER) {
-		/* Keyword -- simple */
-		size->data.length = sizes[size->status - 1];
-	} else if (size->status == CSS_FONT_SIZE_LARGER) {
-		/** \todo Step within table, if appropriate */
-		size->data.length.value =
-				FMUL(parent_size->value, FLTTOFIX(1.2));
-		size->data.length.unit = parent_size->unit;
-	} else if (size->status == CSS_FONT_SIZE_SMALLER) {
-		/** \todo Step within table, if appropriate */
-		size->data.length.value =
-				FMUL(parent_size->value, FLTTOFIX(1.2));
-		size->data.length.unit = parent_size->unit;
-	} else if (size->data.length.unit == CSS_UNIT_EM ||
-			size->data.length.unit == CSS_UNIT_EX) {
-		size->data.length.value =
-			FMUL(size->data.length.value, parent_size->value);
-
-		if (size->data.length.unit == CSS_UNIT_EX) {
-			size->data.length.value = FMUL(size->data.length.value,
-					FLTTOFIX(0.6));
-		}
-
-		size->data.length.unit = parent_size->unit;
-	} else if (size->data.length.unit == CSS_UNIT_PCT) {
-		size->data.length.value = FDIV(FMUL(size->data.length.value,
-				parent_size->value), FLTTOFIX(100));
-		size->data.length.unit = parent_size->unit;
-	}
-#endif
-	size->data.length.unit = CSS_UNIT_EM;
-	size->data.length.value = FMUL(1, 1);
-	size->status = CSS_FONT_SIZE_DIMENSION;
-
-	return CSS_OK;
-}
 
 /******************************************************************************
  * Style selection callbacks                                                  *
@@ -582,6 +510,10 @@ css_error named_ancestor_node(void *pw, void *node,
 {
 	dom_element_named_ancestor_node(node, qname->name,
 			(struct dom_element **)ancestor);
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
+	dom_node_unref(*ancestor);
 
 	return CSS_OK;
 }
@@ -602,6 +534,10 @@ css_error named_parent_node(void *pw, void *node,
 {
 	dom_element_named_parent_node(node, qname->name,
 			(struct dom_element **)parent);
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
+	dom_node_unref(*parent);
 
 	return CSS_OK;
 }
@@ -636,6 +572,9 @@ css_error named_sibling_node(void *pw, void *node,
 
 		err = dom_node_get_node_type(n, &type);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			return CSS_OK;
 		}
@@ -645,10 +584,16 @@ css_error named_sibling_node(void *pw, void *node,
 
 		err = dom_node_get_previous_sibling(n, &prev);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			return CSS_OK;
 		}
 
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node_unref(n);
 		n = prev;
 	}
@@ -658,10 +603,16 @@ css_error named_sibling_node(void *pw, void *node,
 
 		err = dom_node_get_node_name(n, &name);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			return CSS_OK;
 		}
 
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node_unref(n);
 
 		if (dom_string_caseless_lwc_isequal(name, qname->name)) {
@@ -704,6 +655,9 @@ css_error named_generic_sibling_node(void *pw, void *node,
 
 		err = dom_node_get_node_type(n, &type);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			return CSS_OK;
 		}
@@ -711,6 +665,9 @@ css_error named_generic_sibling_node(void *pw, void *node,
 		if (type == DOM_ELEMENT_NODE) {
 			err = dom_node_get_node_name(n, &name);
 			if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 				dom_node_unref(n);
 				return CSS_OK;
 			}
@@ -718,6 +675,9 @@ css_error named_generic_sibling_node(void *pw, void *node,
 			if (dom_string_caseless_lwc_isequal(name,
 					qname->name)) {
 				dom_string_unref(name);
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 				dom_node_unref(n);
 				*sibling = n;
 				break;
@@ -727,10 +687,16 @@ css_error named_generic_sibling_node(void *pw, void *node,
 
 		err = dom_node_get_previous_sibling(n, &prev);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			return CSS_OK;
 		}
 
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node_unref(n);
 		n = prev;
 	}
@@ -751,6 +717,10 @@ css_error named_generic_sibling_node(void *pw, void *node,
 css_error parent_node(void *pw, void *node, void **parent)
 {
 	dom_element_parent_node(node, (struct dom_element **)parent);
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
+	dom_node_unref(*parent);
 
 	return CSS_OK;
 }
@@ -783,6 +753,9 @@ css_error sibling_node(void *pw, void *node, void **sibling)
 
 		err = dom_node_get_node_type(n, &type);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			return CSS_OK;
 		}
@@ -792,16 +765,25 @@ css_error sibling_node(void *pw, void *node, void **sibling)
 
 		err = dom_node_get_previous_sibling(n, &prev);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			return CSS_OK;
 		}
 
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node_unref(n);
 		n = prev;
 	}
 
 	if (n != NULL) {
 		/** \todo Sort out reference counting */
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node_unref(n);
 
 		*sibling = n;
@@ -1354,7 +1336,9 @@ css_error node_is_root(void *pw, void *node, bool *match)
 
 	if (parent != NULL) {
 		err = dom_node_get_node_type(parent, &type);
-
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node_unref(parent);
 
 		if (err != DOM_NO_ERR)
@@ -1435,6 +1419,9 @@ css_error node_count_siblings(void *pw, void *n, bool same_name,
 	}
 
 	if (after) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node *node = dom_node_ref(n);
 		dom_node *next;
 
@@ -1442,13 +1429,18 @@ css_error node_count_siblings(void *pw, void *n, bool same_name,
 			exc = dom_node_get_next_sibling(node, &next);
 			if ((exc != DOM_NO_ERR))
 				break;
-
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(node);
 			node = next;
 
 			cnt += node_count_siblings_check(node, same_name, node_name);
 		} while (node != NULL);
 	} else {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node *node = dom_node_ref(n);
 		dom_node *next;
 
@@ -1456,7 +1448,9 @@ css_error node_count_siblings(void *pw, void *n, bool same_name,
 			exc = dom_node_get_previous_sibling(node, &next);
 			if ((exc != DOM_NO_ERR))
 				break;
-
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(node);
 			node = next;
 
@@ -1499,6 +1493,9 @@ css_error node_is_empty(void *pw, void *node, bool *match)
 		dom_node_type ntype;
 		err = dom_node_get_node_type(n, &ntype);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			return CSS_BADPARM;
 		}
@@ -1506,15 +1503,24 @@ css_error node_is_empty(void *pw, void *node, bool *match)
 		if (ntype == DOM_ELEMENT_NODE ||
 		    ntype == DOM_TEXT_NODE) {
 			*match = false;
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			break;
 		}
 
 		err = dom_node_get_next_sibling(n, &next);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			return CSS_BADPARM;
 		}
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node_unref(n);
 		n = next;
 	}
@@ -1820,7 +1826,8 @@ css_error ua_default_for_property(void *pw, uint32_t property, css_hint *hint)
 	return CSS_OK;
 }
 
-css_error set_libcss_node_data(void *pw, void *node, void *libcss_node_data)
+css_error
+set_libcss_node_data(void *pw, void *node, void *libcss_node_data)
 {
 	dom_node *n = node;
 	dom_exception err;
@@ -1834,12 +1841,19 @@ css_error set_libcss_node_data(void *pw, void *node, void *libcss_node_data)
 	if (err != DOM_NO_ERR) {
 		return CSS_NOMEM;
 	}
+
+	if (old_node_data) {
+		(void)css_libcss_node_data_handler(&selection_handler,
+				CSS_NODE_DELETED,
+				NULL, node, NULL, old_node_data);
+	}
 	//assert(old_node_data == NULL);
 
 	return CSS_OK;
 }
 
-css_error get_libcss_node_data(void *pw, void *node, void **libcss_node_data)
+css_error
+get_libcss_node_data(void *pw, void *node, void **libcss_node_data)
 {
 	dom_node *n = node;
 	dom_exception err;
@@ -2063,7 +2077,7 @@ select_css(struct html_context *html_context, struct html_element *html_element)
 	css_media media = {
 		.type = CSS_MEDIA_SCREEN,
 	};
-	css_unit_ctx unit_len_ctx;
+	css_unit_ctx unit_len_ctx = {0};
 	unit_len_ctx.viewport_width  = 800; // TODO
 	unit_len_ctx.viewport_height = 600; // TODO
 	unit_len_ctx.device_dpi = F_90; //device_dpi;
@@ -2127,6 +2141,10 @@ select_css(struct html_context *html_context, struct html_element *html_element)
 		return;
 	}
 
+	if (!style->styles[CSS_PSEUDO_ELEMENT_NONE]) {
+		goto end;
+	}
+
 	color_type = css_computed_color(
 		style->styles[CSS_PSEUDO_ELEMENT_NONE],
 		&color_shade);
@@ -2173,6 +2191,9 @@ select_css(struct html_context *html_context, struct html_element *html_element)
 	bool is_root = (root == el);
 
 	if (root) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node_unref(root);
 	}
 
@@ -2188,6 +2209,7 @@ select_css(struct html_context *html_context, struct html_element *html_element)
 		apply_text_align(html_context, html_element, text_align);
 	}
 
+end:
 	code = css_select_results_destroy(style);
 	if (code != CSS_OK) {
 		fprintf(stderr, "css_computed_style_destroy code=%d\n", code);
@@ -2332,6 +2354,9 @@ parse_css(struct html_context *html_context, char *name)
 		dom_node_type ntype;
 		err = dom_node_get_node_type(n, &ntype);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			goto end;
 		}
@@ -2353,9 +2378,15 @@ parse_css(struct html_context *html_context, char *name)
 
 		err = dom_node_get_next_sibling(n, &next);
 		if (err != DOM_NO_ERR) {
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 			dom_node_unref(n);
 			goto end;
 		}
+#ifdef ECMASCRIPT_DEBUG
+fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
+#endif
 		dom_node_unref(n);
 		n = next;
 	}
@@ -2386,3 +2417,125 @@ import_css2(struct html_context *html_context, struct uri *uri)
 //		css->import_level--;
 	}
 }
+
+void *
+el_match_selector(const char *selector, void *node)
+{
+	struct string text;
+	css_error code;
+	//size_t size;
+	//uint32_t count;
+	css_stylesheet_params params;
+	css_stylesheet *sheet = NULL;
+	css_select_ctx *select_ctx = NULL;
+	css_select_results *style = NULL;
+	uint8_t color_type;
+	css_color color_shade;
+
+	css_media media = {
+		.type = CSS_MEDIA_SCREEN,
+	};
+
+	css_unit_ctx unit_len_ctx = {0};
+	unit_len_ctx.viewport_width  = 800; // TODO
+	unit_len_ctx.viewport_height = 600; // TODO
+	unit_len_ctx.device_dpi = F_90; //device_dpi;
+
+	/** \todo Change nsoption font sizes to px. */
+///	f_size = FDIV(FMUL(F_96, FDIV(INTTOFIX(nsoption_int(font_size)), F_10)), F_72);
+///	f_min  = FDIV(FMUL(F_96, FDIV(INTTOFIX(nsoption_int(font_min_size)), F_10)), F_72);
+
+	unsigned int f_size = FDIV(FMUL(F_96, FDIV(INTTOFIX(50), F_10)), F_72); // TODO
+	unsigned int f_min  = FDIV(FMUL(F_96, FDIV(INTTOFIX(50), F_10)), F_72); // TODO
+
+	unit_len_ctx.font_size_default = f_size;
+	unit_len_ctx.font_size_minimum = f_min;
+	void *ret = NULL;
+
+	params.params_version = CSS_STYLESHEET_PARAMS_VERSION_1;
+	params.level = CSS_LEVEL_21;
+	params.charset = "UTF-8";
+	params.url = "foo";
+	params.title = "foo";
+	params.allow_quirks = false;
+	params.inline_style = false;
+	params.resolve = resolve_url_empty;
+	params.resolve_pw = NULL;
+	params.import = NULL;
+	params.import_pw = NULL;
+	params.color = NULL;
+	params.color_pw = NULL;
+	params.font = NULL;
+	params.font_pw = NULL;
+
+	if (!selector) {
+		return NULL;
+	}
+	if (!init_string(&text)) {
+		return NULL;
+	}
+	add_to_string(&text, selector);
+	add_to_string(&text, "{color:#123456}");
+
+	/* create a stylesheet */
+	code = css_stylesheet_create(&params, &sheet);
+
+	if (code != CSS_OK) {
+		goto empty;
+	}
+	code = css_stylesheet_append_data(sheet, (const uint8_t *) text.source, text.length);
+
+	if (code != CSS_OK && code != CSS_NEEDDATA) {
+		goto empty;
+	}
+	code = css_stylesheet_data_done(sheet);
+
+	if (code != CSS_OK) {
+		goto empty;
+	}
+	//code = css_stylesheet_size(sheet, &size);
+
+	/* prepare a selection context containing the stylesheet */
+	code = css_select_ctx_create(&select_ctx);
+
+	if (code != CSS_OK) {
+		goto empty;
+	}
+	code = css_select_ctx_append_sheet(select_ctx, sheet, CSS_ORIGIN_AUTHOR, NULL);
+
+	if (code != CSS_OK) {
+		goto empty;
+	}
+//	code = css_select_ctx_count_sheets(select_ctx, &count);
+//
+//	if (code != CSS_OK) {
+//		goto empty;
+//	}
+	code = css_select_style(select_ctx, node, &unit_len_ctx, &media, NULL, &selection_handler, 0, &style);
+
+	if (code != CSS_OK) {
+		goto empty;
+	}
+	color_type = css_computed_color(style->styles[CSS_PSEUDO_ELEMENT_NONE], &color_shade);
+
+	if (color_type && color_shade == 0xff123456) {
+		ret = node;
+	}
+
+empty:
+
+	if (style) {
+		css_select_results_destroy(style);
+	}
+
+	if (select_ctx) {
+		css_select_ctx_destroy(select_ctx);
+	}
+
+	if (sheet) {
+		css_stylesheet_destroy(sheet);
+	}
+	done_string(&text);
+
+	return ret;
+}
