@@ -105,7 +105,6 @@ struct codepage_desc {
  * compiler emits global definitions for the symbols so that the
  * functions can be called from other translation units.  */
 NONSTATIC_INLINE char *encode_utf8(unicode_val_T u);
-NONSTATIC_INLINE int utf8charlen(const char *p);
 NONSTATIC_INLINE unicode_val_T utf8_to_unicode(char **string,
 					       const char *end);
 
@@ -869,69 +868,6 @@ cp_to_unicode(int codepage, char **string, const char *end)
 	return ret;
 }
 
-
-#ifdef CONFIG_COMBINE
-unicode_val_T last_combined = UCS_BEGIN_COMBINED - 1;
-unicode_val_T **combined;
-struct hash *combined_hash;
-
-unicode_val_T
-get_combined(unicode_val_T *data, int length)
-{
-	struct hash_item *item;
-	unicode_val_T *key;
-	int i, indeks;
-
-	assert(length >= 1 && length <= UCS_MAX_LENGTH_COMBINED);
-	if_assert_failed return UCS_NO_CHAR;
-
-	if (!combined_hash) combined_hash = init_hash8();
-	if (!combined_hash) return UCS_NO_CHAR;
-	item = get_hash_item(combined_hash, (char *)data, length * sizeof(*data));
-
-	if (item) return (unicode_val_T)(intptr_t)item->value;
-	if (last_combined >= UCS_END_COMBINED) return UCS_NO_CHAR;
-
-	key = (unicode_val_T *)mem_alloc((length + 1) * sizeof(*key));
-	if (!key) return UCS_NO_CHAR;
-	for (i = 0; i < length; i++)
-		key[i] = data[i];
-	key[i] = UCS_END_COMBINED;
-
-	last_combined++;
-	indeks = last_combined - UCS_BEGIN_COMBINED;
-
-	combined = (unicode_val_T **)mem_realloc(combined, sizeof(*combined) * (indeks + 1));
-	if (!combined) {
-		mem_free(key);
-		last_combined--;
-		return UCS_NO_CHAR;
-	}
-	combined[indeks] = key;
-	item = add_hash_item(combined_hash, (char *)key,
-			     length * sizeof(*data), (void *)(intptr_t)(last_combined));
-	if (!item) {
-		last_combined--;
-		mem_free(key);
-		return UCS_NO_CHAR;
-	}
-	return last_combined;
-}
-
-void
-free_combined()
-{
-	int i, end = last_combined - UCS_BEGIN_COMBINED + 1; 
-
-	if (combined_hash)
-		free_hash(&combined_hash);
-	for (i = 0; i < end; i++)
-		mem_free(combined[i]);
-	mem_free_if(combined);
-}
-#endif /* CONFIG_COMBINE */
-
-
 static void
 add_utf8(struct conv_table *ct, unicode_val_T u, const char *str)
 {
