@@ -231,7 +231,10 @@ init_gettext(struct module *module)
 static void
 done_gettext(struct module *module)
 {
-	mem_free_set(&LANGUAGE, NULL);
+	if (LANGUAGE) {
+		free(LANGUAGE);
+		LANGUAGE = NULL;
+	}
 }
 
 struct module gettext_module = struct_module(
@@ -241,6 +244,7 @@ struct module gettext_module = struct_module(
 	/* submodules: */	NULL,
 	/* data: */		NULL,
 	/* init: */		init_gettext,
-	/* done: */		done_gettext
+	/* done: */		done_gettext,
+	/* getname: */	NULL
 );
 
