@@ -37,7 +37,7 @@
 #define USE_LIBEVENT
 #endif
 
-#if (defined(HAVE_EVENT2_EVENT_H) || defined(HAVE_EV_EVENT_H) || defined(HAVE_LIBEV_EVENT_H)) && defined(HAVE_LIBEVENT) && !defined(OPENVMS) && !defined(DOS)
+#if (defined(HAVE_EVENT2_EVENT_H) || defined(HAVE_EV_EVENT_H)) && defined(HAVE_LIBEVENT) && !defined(OPENVMS) && !defined(DOS)
 #if defined(HAVE_EVENT2_EVENT_H)
 #include <event2/event.h>
 #elif defined(HAVE_EV_EVENT_H)
@@ -50,13 +50,6 @@
 #include <sys/select.h>
 #endif
 
-#define EINTRLOOPX(ret_, call_, x_)			\
-do {							\
-	(ret_) = (call_);				\
-} while ((ret_) == (x_) && errno == EINTR)
-
-#define EINTRLOOP(ret_, call_)	EINTRLOOPX(ret_, call_, -1)
-
 #ifdef CONFIG_LIBCURL
 #include <curl/curl.h>
 #endif
@@ -86,6 +79,13 @@ do {							\
 #endif
 
 #if defined(CONFIG_LIBEVENT) && defined(CONFIG_LIBCURL)
+
+#ifdef CONFIG_OS_WIN32
+#define SOCK_SHIFT 1024
+#else
+#define SOCK_SHIFT 0
+#endif
+
 /* Information associated with a specific easy handle */
 typedef struct _ConnInfo
 {
@@ -108,7 +108,6 @@ typedef struct _SockInfo
 
 GlobalInfo g;
 
-
 #define mycase(code) \
 	case code: s = __STRING(code)
 
@@ -231,7 +230,7 @@ setsock(SockInfo *f, curl_socket_t s, CURL *e, int act, GlobalInfo *g)
 	if (event_initialized(&f->ev)) {
 		event_del(&f->ev);
 	}
-	event_assign(&f->ev, g->evbase, f->sockfd, kind, event_cb, g);
+	event_assign(&f->ev, g->evbase, f->sockfd + SOCK_SHIFT, kind, event_cb, g);
 	event_add(&f->ev, NULL);
 }
 
@@ -521,9 +520,12 @@ mcode_or_die(const char *where, CURLMcode code)
 	}
 }
 
+#endif
+
+#ifdef CONFIG_LIBCURL
 /* Called by libevent when our timeout expires */
 static void
-timer_cb(void *userp)
+timer_cb_select(void *userp)
 {
 	GlobalInfo *g = (GlobalInfo *)userp;
 	CURLMcode rc;
@@ -535,29 +537,27 @@ timer_cb(void *userp)
 
 /* Update the event timer after curl_multi library calls */
 static int
-multi_timer_cb(CURLM *multi, long timeout_ms, GlobalInfo *g)
+multi_timer_cb_select(CURLM *multi, long timeout_ms, GlobalInfo *g)
 {
 	(void)multi;
 	//fprintf(stderr, "multi_timer_cb: Setting timeout to %ld ms\n", timeout_ms);
 
 	/*
-	 * if timeout_ms is -1, just delete the timer
+	 * if timeout_ms is -1, just let the timer to expire.
 	 *
 	 * For all other values of timeout_ms, this should set or *update* the timer
 	 * to the new value
 	 */
 
-	if (timeout_ms == -1) {
-		kill_timer(&g->tim);
-	} else { /* includes timeout zero */
-		install_timer(&g->tim, timeout_ms, timer_cb, g);
+	if (timeout_ms != -1) {
+		install_timer(&g->tim, timeout_ms, timer_cb_select, g);
 	}
 
 	return 0;
 }
 
 static void
-event_read_cb(void *userp)
+event_read_cb_select(void *userp)
 {
 	SockInfo *f = (SockInfo *)userp;
 	GlobalInfo *g = (GlobalInfo*)f->global;
@@ -571,7 +571,7 @@ event_read_cb(void *userp)
 }
 
 static void
-event_write_cb(void *userp)
+event_write_cb_select(void *userp)
 {
 	SockInfo *f = (SockInfo *)userp;
 	GlobalInfo *g = (GlobalInfo*)f->global;
@@ -586,7 +586,7 @@ event_write_cb(void *userp)
 
 /* Clean up the SockInfo structure */
 static void
-remsock(SockInfo *f)
+remsock_select(SockInfo *f)
 {
 	//fprintf(stderr, "remsock f=%p\n", f);
 	if (f) {
@@ -599,7 +599,7 @@ remsock(SockInfo *f)
 
 /* Assign information to a SockInfo structure */
 static void
-setsock(SockInfo *f, curl_socket_t s, CURL *e, int act, GlobalInfo *g)
+setsock_select(SockInfo *f, curl_socket_t s, CURL *e, int act, GlobalInfo *g)
 {
 	int in = act & CURL_POLL_IN;
 	int out = act & CURL_POLL_OUT;
@@ -608,24 +608,24 @@ setsock(SockInfo *f, curl_socket_t s, CURL *e, int act, GlobalInfo *g)
 	f->action = act;
 	f->easy = e;
 
-	set_handlers(s + SOCK_SHIFT, in ? event_read_cb : NULL, out ? event_write_cb : NULL, NULL, f);
+	set_handlers(s + SOCK_SHIFT, in ? event_read_cb_select : NULL, out ? event_write_cb_select : NULL, NULL, f);
 }
 
 /* Initialize a new SockInfo structure */
 static void
-addsock(curl_socket_t s, CURL *easy, int action, GlobalInfo *g)
+addsock_select(curl_socket_t s, CURL *easy, int action, GlobalInfo *g)
 {
 	//fprintf(stderr, "addsock easy=%p\n", easy);
 
 	SockInfo *fdp = mem_calloc(1, sizeof(SockInfo));
 	fdp->global = g;
-	setsock(fdp, s, easy, action, g);
+	setsock_select(fdp, s, easy, action, g);
 	curl_multi_assign(g->multi, s, fdp);
 }
 
 /* CURLMOPT_SOCKETFUNCTION */
 static int
-sock_cb(CURL *e, curl_socket_t s, int what, void *cbp, void *sockp)
+sock_cb_select(CURL *e, curl_socket_t s, int what, void *cbp, void *sockp)
 {
 	GlobalInfo *g = (GlobalInfo*) cbp;
 	SockInfo *fdp = (SockInfo*) sockp;
@@ -635,14 +635,14 @@ sock_cb(CURL *e, curl_socket_t s, int what, void *cbp, void *sockp)
 
 	if (what == CURL_POLL_REMOVE) {
 		//fprintf(stderr, "\n");
-		remsock(fdp);
+		remsock_select(fdp);
 	} else {
 		if (!fdp) {
 			//fprintf(stderr, "Adding data: %s\n", whatstr[what]);
-			addsock(s, e, what, g);
+			addsock_select(s, e, what, g);
 		} else {
 			//fprintf(stderr, "Changing action from %s to %s\n", whatstr[fdp->action], whatstr[what]);
-			setsock(fdp, s, e, what, g);
+			setsock_select(fdp, s, e, what, g);
 		}
 	}
 
@@ -786,12 +786,6 @@ int event_enabled = 0;
 #define sizeof_struct_event		(event_get_struct_event_size())
 #endif
 
-static inline
-struct event *timer_event(struct timer *tm)
-{
-	return (struct event *)((char *)tm - sizeof_struct_event);
-}
-
 #ifdef HAVE_EVENT_BASE_SET
 struct event_base *event_base;
 #endif
@@ -869,6 +863,10 @@ enable_libevent(void)
 		return;
 #endif
 
+#if !defined(EVENT__DISABLE_MM_REPLACEMENT) && defined(CONFIG_DEBUG)
+	event_set_mem_functions(el_libevent_malloc, el_libevent_realloc, el_libevent_free);
+#endif
+
 #if defined(HAVE_EVENT_CONFIG_SET_FLAG)
 	{
 		struct event_config *cfg;
@@ -895,6 +893,7 @@ enable_libevent(void)
 #else
 	event_init();
 #endif
+
 	event_enabled = 1;
 
 	for (i = 0; i < w_max; i++)
@@ -945,7 +944,7 @@ do_event_loop(int flags)
 select_handler_T
 get_handler(int fd, enum select_handler_type tp)
 {
-	if (fd >= w_max) {
+	if (fd >= w_max || fd < 0) {
 		return NULL;
 	}
 
@@ -962,7 +961,7 @@ get_handler(int fd, enum select_handler_type tp)
 void *
 get_handler_data(int fd)
 {
-	if (fd >= w_max) {
+	if (fd >= w_max || fd < 0) {
 		return NULL;
 	}
 
@@ -973,6 +972,9 @@ void
 set_handlers(int fd, select_handler_T read_func, select_handler_T write_func,
 	     select_handler_T error_func, void *data)
 {
+	if (fd < 0) {
+		return;
+	}
 #ifndef CONFIG_OS_WIN32
 	assertm(fd >= 0 && fd < FD_SETSIZE,
 		"set_handlers: handle %d >= FD_SETSIZE %d",
@@ -1088,11 +1090,35 @@ try_redraw_all_terminals(void)
 	redraw_all_terminals();
 }
 
+#ifndef NO_SIGNAL_HANDLERS
+static void
+clear_events(int h, int blocking)
+{
+#if !defined(O_NONBLOCK) && !defined(FIONBIO)
+	blocking = 1;
+#endif
+	while (blocking ? can_read(h) : 1) {
+		unsigned char c[64];
+		int rd;
+		EINTRLOOP(rd, (int)read(h, c, sizeof c));
+		if (rd != sizeof c) break;
+	}
+}
+
+pid_t signal_pid;
+int signal_pipe[2] = { -1, -1 };
+
+static void
+clear_events_ptr(void *handle)
+{
+	clear_events((int)(intptr_t)handle, 0);
+}
+
+#endif
 
 void
 select_loop(void (*init)(void))
 {
-
 	timeval_T last_time;
 	int select_errors = 0;
 
@@ -1104,9 +1130,18 @@ select_loop(void (*init)(void))
 	timeval_now(&last_time);
 #ifdef SIGPIPE
 	signal(SIGPIPE, SIG_IGN);
+#endif
+#if !defined(NO_SIGNAL_HANDLERS)
+	signal_pid = getpid();
+
+	if (c_pipe(signal_pipe)) {
+		elinks_internal("ERROR: can't create pipe for signal handling");
+	}
+	set_nonblocking_fd(signal_pipe[0]);
+	set_nonblocking_fd(signal_pipe[1]);
+	set_handlers(signal_pipe[0], clear_events_ptr, NULL, NULL, (void *)(intptr_t)signal_pipe[0]);
 #endif
 	init();
-	check_bottom_halves();
 
 #ifdef USE_LIBEVENT
 	enable_libevent();
@@ -1163,6 +1198,7 @@ select_loop(void (*init)(void))
 
 		/* we do not call any curl_multi_socket*() function yet as we have no handles added! */
 #endif
+		check_bottom_halves();
 		while (!program.terminate) {
 			check_signals();
 			if (1 /*(!F)*/) {
@@ -1204,14 +1240,13 @@ select_loop(void (*init)(void))
 		g.multi = curl_multi_init();
 
 		/* setup the generic multi interface options we want */
-		curl_multi_setopt(g.multi, CURLMOPT_SOCKETFUNCTION, sock_cb);
+		curl_multi_setopt(g.multi, CURLMOPT_SOCKETFUNCTION, sock_cb_select);
 		curl_multi_setopt(g.multi, CURLMOPT_SOCKETDATA, &g);
-		curl_multi_setopt(g.multi, CURLMOPT_TIMERFUNCTION, multi_timer_cb);
+		curl_multi_setopt(g.multi, CURLMOPT_TIMERFUNCTION, multi_timer_cb_select);
 		curl_multi_setopt(g.multi, CURLMOPT_TIMERDATA, &g);
 		/* we do not call any curl_multi_socket*() function yet as we have no handles added! */
 #endif
-
-
+		check_bottom_halves();
 
 	while (!program.terminate) {
 		struct timeval timeout = { 0, 0 };
@@ -1231,10 +1266,8 @@ select_loop(void (*init)(void))
 
 		has_timer = get_next_timer_time(&t);
 		if (!w_max && !has_timer) break;
-		critical_section = 1;
 
 		if (check_signals()) {
-			critical_section = 0;
 			continue;
 		}
 #if 0
@@ -1268,8 +1301,6 @@ select_loop(void (*init)(void))
 			 * might change errno.  */
 			const int errno_from_select = errno;
 
-			critical_section = 0;
-			uninstall_alarm();
 			if (errno_from_select != EINTR) {
 				ERROR(gettext("The call to %s failed: %d (%s)"),
 				      "select()", errno_from_select, (char *) strerror(errno_from_select));
@@ -1281,8 +1312,6 @@ select_loop(void (*init)(void))
 		}
 
 		select_errors = 0;
-		critical_section = 0;
-		uninstall_alarm();
 		check_signals();
 		/*printf("sel: %d\n", n);*/
 		check_timers(&last_time);
