@@ -15,7 +15,7 @@
 #define USE_LIBEVENT
 #endif
 
-#if (defined(HAVE_EVENT2_EVENT_H) || defined(HAVE_EV_EVENT_H) || defined(HAVE_LIBEV_EVENT_H)) && defined(HAVE_LIBEVENT) && !defined(OPENVMS) && !defined(DOS)
+#if (defined(HAVE_EVENT2_EVENT_H) || defined(HAVE_EV_EVENT_H)) && defined(HAVE_LIBEVENT) && !defined(OPENVMS) && !defined(DOS)
 #if defined(HAVE_DEVENT2_EVENT_H)
 #include <event2/event.h>
 #elif defined(HAVE_EV_EVENT_H)
@@ -34,7 +34,7 @@
 #include "util/time.h"
 
 #if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
-#include "ecmascript/timer.h"
+#include "js/timer.h"
 #endif
 
 /* @timers.next points to the timer with the smallest interval,
@@ -156,7 +156,7 @@ install_timer(timer_id_T *id, milliseconds_T delay, void (*func)(void *), void *
 {
 	struct timer *new_timer, *timer;
 
-	assert(id && delay > 0);
+	assert(id && delay >= 0);
 
 #ifdef USE_LIBEVENT
 	char *q = (char *)mem_alloc(sizeof_struct_event + sizeof(struct timer));
