@@ -416,7 +416,7 @@ init_mailcap_map(void)
 	if (!mailcap_map) return NULL;
 
 	/* Try to setup mailcap_path */
-	path = get_mailcap_path();
+	path = (char *)get_mailcap_path();
 	if (!path || !*path) path = getenv("MAILCAP");
 	if (!path) path = (char *)DEFAULT_MAILCAP_PATH;
 
@@ -788,13 +788,30 @@ struct module mailcap_mime_module = struct_module(
 	/* submodules: */	NULL,
 	/* data: */		NULL,
 	/* init: */		init_mailcap,
-	/* done: */		done_mailcap
+	/* done: */		done_mailcap,
+	/* getname: */	NULL
 );
 
 #ifdef TEST_MAILCAP
 
 #include "util/test.h"
 
+struct option *cmdline_options;
+
+#ifdef CONFIG_DEBUG
+union option_value *
+get_opt_(char *a, int, enum option_type b, struct option *c, const char *d, struct session *e)
+{
+	return NULL;
+}
+#else
+union option_value *
+get_opt_(struct option *a , const char *b, struct session *c)
+{
+	return NULL;
+}
+#endif
+
 char *get_ui_clipboard_file(void) { return NULL; }
 
 /* Some ugly shortcuts for getting defined symbols to work. */
@@ -809,13 +826,15 @@ main(int argc, char *argv[])
 {
 	char *format = "description,ask,block,program";
 	int has_gotten = 0;
+	int setenv_display = 0;
 	int i;
 
 	for (i = 1; i < argc; i++) {
 		char *arg = argv[i];
 
-		if (strncmp(arg, "--", 2))
+		if (strncmp(arg, "--", 2)) {
 			break;
+		}
 
 		arg += 2;
 
@@ -826,27 +845,40 @@ main(int argc, char *argv[])
 		} else if (get_test_opt(&arg, "format", &i, argc, argv, "a string")) {
 			format = arg;
 
+		} else if (get_test_opt(&arg, "display", &i, argc, argv, "a string")) {
+			if (strcmp(arg, "1") == 0 || strcmp(arg, "yes") == 0) {
+				setenv_display = 1;
+			}
+
 		} else if (get_test_opt(&arg, "get", &i, argc, argv, "a string")) {
 			struct mime_handler *handler;
 
-			if (has_gotten)
+			if (has_gotten) {
 				printf("\n");
+			}
 			has_gotten = 1;
 			printf("type: %s\n", arg);
-			handler = get_mime_handler_mailcap(arg, 0);
-			if (!handler) continue;
+			handler = get_mime_handler_mailcap(arg, setenv_display);
 
-			if (strstr(format, "description"))
+			if (!handler) {
+				continue;
+			}
+
+			if (strstr(format, "description")) {
 				printf("description: %s\n", handler->description);
+			}
 
-			if (strstr(format, "ask"))
+			if (strstr(format, "ask")) {
 				printf("ask: %d\n", handler->ask);
+			}
 
-			if (strstr(format, "block"))
+			if (strstr(format, "block")) {
 				printf("block: %d\n", handler->block);
+			}
 
-			if (strstr(format, "program"))
+			if (strstr(format, "program")) {
 				printf("program: %s\n", handler->program);
+			}
 
 		} else {
 			die("Unknown argument '%s'", arg - 2);
