@@ -5,8 +5,8 @@
 #endif
 
 #ifdef CONFIG_OPENSSL
-#include <openssl/ssl.h>
 #include <openssl/x509v3.h>
+#include <openssl/ssl.h>
 #define USE_OPENSSL
 #elif defined(CONFIG_NSS_COMPAT_OSSL)
 #include <nss_compat_ossl/nss_compat_ossl.h>
@@ -260,6 +260,7 @@ static int
 match_uri_host_ip(const char *uri_host,
 		  ASN1_OCTET_STRING *cert_host_asn1)
 {
+#if defined(HAVE_INET_PTON) || defined(HAVE_INET_ATON)
 #ifdef HAVE_ASN1_STRING_GET0_DATA
 	const unsigned char *cert_host_addr = ASN1_STRING_get0_data(cert_host_asn1);
 #else
@@ -267,10 +268,11 @@ match_uri_host_ip(const char *uri_host,
 #endif
 
 	struct in_addr uri_host_in;
+#ifdef HAVE_INET_PTON
 #ifdef CONFIG_IPV6
 	struct in6_addr uri_host_in6;
 #endif
-
+#endif
 	/* RFC 5280 defines the iPAddress alternative of GeneralName
 	 * as an OCTET STRING.  Verify that the type is indeed that.
 	 * This is an assertion because, if someone puts the wrong
@@ -311,6 +313,9 @@ match_uri_host_ip(const char *uri_host,
 	default:
 		return 0;
 	}
+#else
+	return 0;
+#endif
 }
 
 /** Verify one certificate in the server certificate chain.
@@ -537,7 +542,7 @@ ssl_connect(struct socket *socket)
 	 * ELinks must just cast the pointer the best it can and hope
 	 * that the conversions match.  */
 	gnutls_transport_set_ptr(*((ssl_t *) socket->ssl),
-				 (gnutls_transport_ptr_t) (longptr_T) socket->fd);
+				 (gnutls_transport_ptr_t) (intptr_t) socket->fd);
 
 	/* TODO: Some certificates fuss. --pasky */
 #endif
