@@ -25,7 +25,7 @@
 #include <unistd.h>
 #include <values.h>
 #ifdef CONFIG_QUICKJS
-#include "ecmascript/quickjs/heartbeat.h"
+#include "js/quickjs/heartbeat.h"
 #endif
 #include "intl/libintl.h"
 #include "main/main.h"
@@ -983,6 +983,21 @@ int dos_select(int n, fd_set *rs, fd_set *ws, fd_set *es, struct timeval *t, int
 	}
 }
 
+long
+os_get_free_mem_in_mib(void)
+{
+	__dpmi_memory_info buffer;
+	int ret = __dpmi_get_memory_information(&buffer);
+
+	if (ret) {
+		__dpmi_free_mem_info info;
+		__dpmi_get_free_memory_information(&info);
+
+		return info.total_number_of_free_pages / 256;
+	}
+	return buffer.total_available_bytes_of_virtual_memory_client / (1024 * 1024);
+}
+
 #ifdef DOS_EXTRA_KEYBOARD
 
 int dos_setraw(int ctl, int save)
