@@ -7,6 +7,10 @@ extern "C" {
 
 #ifdef CONFIG_OS_DOS
 
+#undef CHAR_DIR_SEP
+#define CHAR_DIR_SEP '\\'
+#undef STRING_DIR_SEP
+#define STRING_DIR_SEP "\\"
 
 #define DOS_EXTRA_KEYBOARD
 
