@@ -62,6 +62,7 @@
 #include "elinks.h"
 
 #include "config/options.h"
+#include "main/main.h"
 #include "main/select.h"
 #include "osdep/osdep.h"
 #include "osdep/signals.h"
@@ -73,6 +74,14 @@
 #include "util/string.h"
 
 
+#if !defined(CONFIG_OS_DOS) && !defined(CONFIG_OS_UNIX) && !defined(CONFIG_OS_WIN32)
+long
+os_get_free_mem_in_mib(void)
+{
+	return 0;
+}
+#endif
+
 #ifndef CONFIG_OS_DOS
 /* Set a file descriptor to non-blocking mode. It returns a non-zero value
  * on error. */
@@ -321,7 +330,7 @@ is_xterm(void)
 	if (xt == -1) {
 		char *term = getenv("TERM");
 
-		if (term && !strncmp("xterm", term, 5)) {
+		if (term && strstr(term, "xterm")) {
 			xt = 1;
 		} else {
 			char *wayland = getenv("WAYLAND_DISPLAY");
@@ -875,6 +884,10 @@ done_draw(void)
 int
 get_output_handle(void)
 {
+	if (program.testjs) {
+		return open("/dev/null", O_WRONLY);
+	}
+
 	return 1;
 }
 
@@ -883,7 +896,9 @@ get_ctl_handle(void)
 {
 	static int fd = -1;
 
-	if (isatty(0)) return 0;
+	if (isatty(0)) {
+		return 0;
+	}
 	if (fd < 0) fd = open("/dev/tty", O_RDONLY);
 	return fd;
 }
@@ -959,7 +974,7 @@ elinks_cfmakeraw(struct termios *t)
 	t->c_cc[VTIME] = 0;
 }
 
-#if !defined(CONFIG_MOUSE) || (!defined(CONFIG_GPM) && !defined(CONFIG_SYSMOUSE) && !defined(OS2_MOUSE) && !defined(CONFIG_OS_DOS))
+#if !defined(CONFIG_MOUSE) || (!defined(CONFIG_GPM) && !defined(CONFIG_SYSMOUSE) && !defined(OS2_MOUSE) && !defined(CONFIG_OS_DOS) && !defined(CONFIG_OS_WIN32))
 
 void *
 handle_mouse(int cons, void (*fn)(void *, char *, int),
