@@ -77,19 +77,19 @@ static const char *keymap_2[] = {
 
 #define TRACE(m...)					\
 	do {						\
-		if (get_cmd_opt_int("verbose") == 2)	\
+		if (cmdline_options && (get_cmd_opt_int("verbose") == 2))	\
 			DBG(m);				\
 	} while (0)
 
 static enum fd_types
 what_fd_type(int *fd)
 {
-	if (*fd == (int)GetStdHandle(STD_INPUT_HANDLE)
-	    || *fd == (int)GetStdHandle(STD_OUTPUT_HANDLE)
-	    || *fd == (int)GetStdHandle(STD_ERROR_HANDLE))
+	if (*fd == (int)(intptr_t)GetStdHandle(STD_INPUT_HANDLE)
+	    || *fd == (int)(intptr_t)GetStdHandle(STD_OUTPUT_HANDLE)
+	    || *fd == (int)(intptr_t)GetStdHandle(STD_ERROR_HANDLE))
 		return FDT_TERMINAL;
 
-	if (GetFileType((HANDLE) *fd) == FILE_TYPE_PIPE)
+	if (GetFileType((HANDLE)(intptr_t)*fd) == FILE_TYPE_PIPE)
 		return FDT_PIPE;
 
 	if (*fd >= SOCK_SHIFT) {
@@ -129,25 +129,93 @@ console_key_read(const KEY_EVENT_RECORD *ker, char *buf, int max)
 }
 
 #ifdef CONFIG_MOUSE
+
+struct elwin_mouse {
+	int cons;
+	void (*fn)(void *, char *, int);
+	void *data;
+} elm;
+
 static int
 console_mouse_read(const MOUSE_EVENT_RECORD *mer, char *buf, int max)
 {
-	struct term_event ev;
-	int len;
+	static struct interlink_event_mouse prev_mouse;
+	static int prev_buttons;
+	struct interlink_event ev;
+	struct interlink_event_mouse mouse;
+	int buttons;
+	int change;
+	int wheeled;
+	int16_t wdy;
 
-	if (mer->dwEventFlags || !mer->dwButtonState)  /* not mouse down, or shifts */
+	mouse.x = mer->dwMousePosition.X;
+	mouse.y = mer->dwMousePosition.Y;
+	change = (mouse.x != prev_mouse.x || mouse.y != prev_mouse.y);
+	buttons = mer->dwButtonState & 7;
+
+	if (!change && (buttons == prev_buttons) && !mer->dwEventFlags) {
+		return 0;
+	}
+	prev_buttons = buttons;
+	wheeled = mer->dwEventFlags & MOUSE_WHEELED;
+	wdy = (mer->dwButtonState >> 16);
+
+	if (buttons & FROM_LEFT_1ST_BUTTON_PRESSED)
+		mouse.button = B_LEFT;
+	else if (buttons & FROM_LEFT_2ND_BUTTON_PRESSED)
+		mouse.button = B_MIDDLE;
+	else if (buttons & RIGHTMOST_BUTTON_PRESSED)
+		mouse.button = B_RIGHT;
+	else if (wheeled && wdy < 0)
+		mouse.button = B_WHEEL_DOWN;
+	else if (wheeled && wdy > 0)
+		mouse.button = B_WHEEL_UP;
+	else
 		return 0;
 
-	set_mouse_term_event(&ev,
-			     1 + mer->dwMousePosition.X,
-			     1 + mer->dwMousePosition.Y,
-			     B_DOWN);
+	mouse.button |= B_DOWN;
+	prev_mouse = mouse;
+	set_mouse_interlink_event(&ev, mouse.x, mouse.y, mouse.button);
 
-	len = int_min(max, sizeof(ev));
-	memcpy(buf, &ev, len);
+	if (elm.fn) {
+		elm.fn(elm.data, (char *)&ev, sizeof(ev));
+	}
+	set_mouse_interlink_event(&ev, mouse.x, mouse.y, (mouse.button & ~B_DOWN) | B_UP);
 
-	return len;
+	if (elm.fn) {
+		elm.fn(elm.data, (char *)&ev, sizeof(ev));
+	}
+
+	return 0;
 }
+
+void *
+handle_mouse(int cons, void (*fn)(void *, char *, int),
+	     void *data)
+{
+	elm.cons = cons;
+	elm.fn = fn;
+	elm.data = data;
+
+	return &elm;
+}
+
+void
+unhandle_mouse(void *h)
+{
+	memset(&elm, 0, sizeof(elm));
+}
+
+void
+suspend_mouse(void *h)
+{
+}
+
+void
+resume_mouse(void *h)
+{
+}
+
 #endif
 
 static int
@@ -188,7 +256,7 @@ static int
 console_peek(HANDLE hnd)
 {
 	DWORD num = 0;
-	char  buf[10];
+	char  buf[32];
 	int   rc = 0;
 
 	if (!GetNumberOfConsoleInputEvents(hnd, &num) || num == 0)
@@ -221,7 +289,7 @@ win32_write(int fd, const void *buf, unsigned len)
 		break;
 
 	case FDT_PIPE:
-		WriteFile((HANDLE) fd, buf, len, &written, NULL);
+		WriteFile((HANDLE)(intptr_t)fd, buf, len, &written, NULL);
 		rc = written;
 		if (rc != len)
 			errno = GetLastError();
@@ -230,10 +298,11 @@ win32_write(int fd, const void *buf, unsigned len)
 	case FDT_TERMINAL:
 		if (isatty(STDOUT_FILENO) > 0) {
 #if 0
-			WriteConsole ((HANDLE) fd, buf, len, &written, NULL);
+			WriteConsole ((HANDLE)(intptr_t)fd, buf, len, &written, NULL);
 			rc = written;
+#else
+			rc = VT100_decode((HANDLE)(intptr_t)fd, buf, len);
 #endif
-			rc = VT100_decode((HANDLE) fd, buf, len);
 		} else {
 			/* stdout redirected */
 			rc = write(STDOUT_FILENO, buf, len);
@@ -266,14 +335,14 @@ win32_read(int fd, void *buf, unsigned len)
 		break;
 
 	case FDT_PIPE:
-		if (!ReadFile((HANDLE) fd, buf, len, &Read, NULL))
+		if (!ReadFile((HANDLE)(intptr_t)fd, buf, len, &Read, NULL))
 			rc = -1;
 		else
 			rc = Read;
 		break;
 
 	case FDT_TERMINAL:
-		rc = console_read((HANDLE) fd, buf, len, NULL);
+		rc = console_read((HANDLE)(intptr_t)fd, buf, len, NULL);
 		if (rc == 0)
 			TRACE("read 0 !!");
 		break;
@@ -303,7 +372,7 @@ win32_close(int fd)
 		break;
 
 	case FDT_PIPE:
-		if (CloseHandle((HANDLE)fd))
+		if (CloseHandle((HANDLE)(intptr_t)fd))
 			rc = 0;
 		break;
 
@@ -342,7 +411,7 @@ win32_ioctl(int fd, long option, int *flag)
 
 		mode = flg ? PIPE_NOWAIT : PIPE_WAIT;
 		mode |= PIPE_READMODE_BYTE;
-		SetNamedPipeHandleState((HANDLE) fd, &mode, NULL, NULL);
+		SetNamedPipeHandleState((HANDLE)(intptr_t)fd, &mode, NULL, NULL);
 		rc = 0;
 		break;
 
@@ -491,8 +560,8 @@ win32_pipe(int *fds)
 		return (-1);
 	}
 
-	fds[0] = (int) rd_pipe;
-	fds[1] = (int) wr_pipe;
+	fds[0] = (int)(intptr_t)rd_pipe;
+	fds[1] = (int)(intptr_t)wr_pipe;
 
 	return 0;
 }
@@ -500,7 +569,7 @@ win32_pipe(int *fds)
 static const char *
 timeval_str(const struct timeval *tv)
 {
-	static unsigned char buf[30];
+	static char buf[30];
 
 	snprintf(buf, sizeof(buf), "%ld.%03ld", tv->tv_sec, tv->tv_usec/1000);
 
@@ -549,7 +618,7 @@ static int
 select_read(int fd, struct fd_set *rd)
 {
 	int rc = 0;
-	HANDLE hnd = (HANDLE) fd;
+	HANDLE hnd = (HANDLE)(intptr_t)fd;
 
 	if (GetFileType(hnd) == FILE_TYPE_PIPE) {
 		DWORD read = 0;
@@ -564,7 +633,7 @@ select_read(int fd, struct fd_set *rd)
 			rc++;
 		}
 	} else {
-		hnd = (HANDLE) _get_osfhandle(fd);
+		hnd = (HANDLE)_get_osfhandle(fd);
 		if (WaitForSingleObject(hnd, 0) == WAIT_OBJECT_0) {
 			FD_SET (fd, rd);
 			rc++;
@@ -636,7 +705,7 @@ int win32_select (int num_fds, struct fd_set *rd, struct fd_set *wr,
 		struct fd_set *ex, struct timeval *tv)
 {
 	struct fd_set tmp_rd, tmp_ex;
-	struct timeval expiry, start_time;
+	struct timeval expiry = {0};
 	int    fd, rc;
 	BOOL   expired = FALSE;
 
@@ -651,6 +720,7 @@ int win32_select (int num_fds, struct fd_set *rd, struct fd_set *wr,
 	FD_ZERO(&tmp_ex);
 
 	if (tv) {
+		struct timeval start_time;
 		gettimeofday(&start_time, NULL);
 
 		expiry.tv_sec  = start_time.tv_sec  + tv->tv_sec;
