@@ -3,6 +3,10 @@
 
 #ifdef CONFIG_OS_WIN32
 
+#ifdef HAVE_WINSOCK2_H
+#include <winsock2.h>
+#endif
+
 #ifdef HAVE_WINDOWS_H
 #include <windows.h>
 #endif
