@@ -27,15 +27,31 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <locale.h>
+#include <share.h>
+#include <sys/stat.h>
 
 #include "elinks.h"
 
+#ifdef CONFIG_QUICKJS
+#include "js/quickjs/heartbeat.h"
+#endif
+
 #include "main/select.h"
 #include "main/timer.h"
 #include "osdep/win32/win32.h"
 #include "osdep/osdep.h"
 #include "terminal/terminal.h"
 
+#ifdef CONFIG_QUICKJS
+static HANDLE hTimer = NULL;
+static HANDLE hTimerQueue = NULL;
+
+VOID CALLBACK
+TimerRoutine(PVOID lpParam, BOOLEAN TimerOrWaitFired)
+{
+	check_heartbeats(NULL);
+}
+#endif
 
 void
 init_osdep(void)
@@ -66,9 +82,34 @@ init_osdep(void)
 #endif
 }
 
+#ifdef CONFIG_QUICKJS
+void
+create_timer_queue(void)
+{
+	hTimerQueue = CreateTimerQueue();
+
+	if (hTimerQueue != NULL) {
+		(void)CreateTimerQueueTimer(&hTimer, hTimerQueue,
+		(WAITORTIMERCALLBACK)TimerRoutine, NULL , 1000, 1000, 0);
+	}
+}
+
+static void
+delete_timer_queue(void)
+{
+	if (hTimerQueue) {
+		DeleteTimerQueueEx(hTimerQueue, NULL);
+		hTimerQueue = NULL;
+	}
+}
+#endif
+
 void
 terminate_osdep(void)
 {
+#ifdef CONFIG_QUICKJS
+	delete_timer_queue();
+#endif
 }
 
 int
@@ -164,7 +205,7 @@ int
 exe(char *path)
 {
 	int rc;
-	char *shell = get_shell();
+	const char *shell = get_shell();
 	char *x = *path != '"' ? " /c start /wait " : " /c start /wait \"\" ";
 	char *p = malloc((strlen(shell) + strlen(x) + strlen(path)) * 2 + 1);
 
@@ -240,17 +281,13 @@ get_input_handle(void)
 	static HANDLE hStdIn = INVALID_HANDLE_VALUE;
 
 	if (hStdIn == INVALID_HANDLE_VALUE) {
-		DWORD dwMode;
-
 		SetConsoleTitle("ELinks - Console mode browser");
 
 		hStdIn = GetStdHandle(STD_INPUT_HANDLE);
-		GetConsoleMode(hStdIn, &dwMode);
-		dwMode &= ~(ENABLE_LINE_INPUT |	ENABLE_ECHO_INPUT);
-		dwMode |= (ENABLE_WINDOW_INPUT | ENABLE_MOUSE_INPUT);
-		SetConsoleMode(hStdIn, dwMode);
+		SetConsoleMode(hStdIn, ENABLE_EXTENDED_FLAGS);
+		SetConsoleMode(hStdIn, (ENABLE_WINDOW_INPUT | ENABLE_MOUSE_INPUT));
 	}
-	return (int) hStdIn;
+	return (int)(intptr_t)hStdIn;
 }
 
 int
@@ -258,9 +295,14 @@ get_output_handle(void)
 {
 	static HANDLE hStdOut = INVALID_HANDLE_VALUE;
 
-	if (hStdOut == INVALID_HANDLE_VALUE)
+	if (hStdOut == INVALID_HANDLE_VALUE) {
+		DWORD dwMode;
+
 		hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);
-	return (int) hStdOut;
+		GetConsoleMode(hStdOut, &dwMode);
+		SetConsoleMode(hStdOut, dwMode);
+	}
+	return (int)(intptr_t)hStdOut;
 }
 
 int
@@ -278,19 +320,32 @@ gettimeofday(struct timeval* p, void* tz)
 	return 0;
 }
 
-
 int
-mkstemp(char *template_)
+mkstemp(char *template_name)
 {
-	char pathname[MAX_PATH];
+	int i, j, fd, len, index;
 
- 	/* Get the directory for temp files */
-	GetTempPath(MAX_PATH, pathname);
+	static const char letters[] = "abcdefghijklmnopqrstuvxyzABCDEFGHIJKLMNOPQRSTUVXYZ0123456789";
 
-	/* Create a temporary file. */
-	GetTempFileName(pathname, "ABC", 0, template_);
+	if (template_name == NULL || (len = strlen(template_name)) < 6
+	    || memcmp(template_name + (len - 6), "XXXXXX", 6)) {
+		errno = EINVAL;
+		return -1;
+	}
 
-	return open(template_, O_WRONLY | O_BINARY | O_EXCL);
+	for (index = len - 6; index > 0 && template_name[index - 1] == 'X'; index--);
+
+	for (i = 0; i >= 0; i++) {
+		for (j = index; j < len; j++) {
+			template_name[j] = letters[rand() % 62];
+		}
+		fd = _sopen(template_name,
+		    _O_RDWR | _O_CREAT | _O_EXCL | _O_BINARY,
+		    _SH_DENYRW, _S_IREAD | _S_IWRITE);
+		if (fd != -1) return fd;
+		if (fd == -1 && errno != EEXIST) return -1;
+	}
+	return -1;
 }
 
 int
@@ -335,3 +390,13 @@ user_appdata_directory(void)
 	else
 		return NULL;
 }
+
+long
+os_get_free_mem_in_mib(void)
+{
+	MEMORYSTATUSEX statex;
+	statex.dwLength = sizeof(statex);
+	GlobalMemoryStatusEx(&statex);
+
+	return statex.ullAvailPhys / (1024 * 1024);
+}
