@@ -3,6 +3,14 @@
 
 #include "protocol/protocol.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern protocol_handler_T about_protocol_handler;
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
