@@ -49,7 +49,7 @@
 #include "protocol/common.h"
 #include "protocol/curl/ftpes.h"
 #include "protocol/ftp/ftp.h"
-#include "protocol/ftp/parse.h"
+#include "protocol/ftpparse.h"
 #include "protocol/uri.h"
 #include "util/conv.h"
 #include "util/error.h"
@@ -80,6 +80,18 @@ union option_info ftp_options[] = {
 	INIT_OPT_BOOL("protocol.ftp", N_("Use libcurl"),
 		"use_curl", OPT_ZERO, 0,
 		N_("Use libcurl implementation of ftp.")),
+
+	INIT_OPT_LONG("protocol.ftp", N_("Curl max receive speed"),
+		"curl_max_recv_speed", OPT_ZERO, 0, LONG_MAX, 0,
+		N_("Max download speed in bytes/s. 0 means unlimited.")),
+
+	INIT_OPT_LONG("protocol.ftp", N_("Curl max send speed"),
+		"curl_max_send_speed", OPT_ZERO, 0, LONG_MAX, 0,
+		N_("Max upload speed in bytes/s. 0 means unlimited.")),
+
+	INIT_OPT_STRING("protocol.ftp", N_("Curl TLS13 ciphers"),
+		"curl_tls13_ciphers", OPT_ZERO, "",
+		N_("Colon separated list of TLS13 ciphers. See https://curl.se/docs/ssl-ciphers.html")),
 #endif
 
 	INIT_OPT_BOOL("protocol.ftp", N_("Use passive mode (IPv4)"),
@@ -103,7 +115,8 @@ struct module ftp_protocol_module = struct_module(
 	/* submodules: */	NULL,
 	/* data: */		NULL,
 	/* init: */		NULL,
-	/* done: */		NULL
+	/* done: */		NULL,
+	/* getname: */	NULL
 );
 
 
