@@ -1,4 +1,3 @@
-
 #ifndef EL__PROTOCOL_GEMINI_GEMINI_H
 #define EL__PROTOCOL_GEMINI_GEMINI_H
 
@@ -11,8 +10,22 @@ extern "C" {
 
 struct connection;
 struct read_buffer;
+struct session;
 struct socket;
 
+struct gemini_connection_info {
+	char *prompt;
+	int code;
+};
+
+struct gemini_error_info {
+	int code;
+	struct uri *uri;
+	struct session *ses;
+	char *prompt;
+	char value[MAX_STR_LEN];
+};
+
 extern struct module gemini_protocol_module;
 
 #ifdef CONFIG_GEMINI
