@@ -201,6 +201,14 @@ static union option_info http_options[] = {
 	INIT_OPT_BOOL("protocol.http", N_("Use libcurl"),
 		"use_curl", OPT_ZERO, 0,
 		N_("Use libcurl implementation of http(s).")),
+
+	INIT_OPT_LONG("protocol.http", N_("Curl max receive speed"),
+		"curl_max_recv_speed", OPT_ZERO, 0, LONG_MAX, 0,
+		N_("Max download speed in bytes/s. 0 means unlimited.")),
+
+	INIT_OPT_LONG("protocol.http", N_("Curl max send speed"),
+		"curl_max_send_speed", OPT_ZERO, 0, LONG_MAX, 0,
+		N_("Max upload speed in bytes/s. 0 means unlimited.")),
 #endif
 	/* OSNews.com is supposed to be relying on the textmode token, at least. */
 	INIT_OPT_STRING("protocol.http", N_("User-agent identification"),
@@ -227,6 +235,12 @@ static union option_info http_options[] = {
 		"https", OPT_ZERO,
 		N_("HTTPS-specific options.")),
 
+#if defined(CONFIG_LIBCURL)
+	INIT_OPT_STRING("protocol.https", N_("Curl TLS13 ciphers"),
+		"curl_tls13_ciphers", OPT_ZERO, "",
+		N_("Colon separated list of TLS13 ciphers. See https://curl.se/docs/ssl-ciphers.html")),
+#endif
+
 	INIT_OPT_TREE("protocol.https", N_("Proxy configuration"),
 		"proxy", OPT_ZERO,
 		N_("HTTPS proxy configuration.")),
@@ -236,6 +250,7 @@ static union option_info http_options[] = {
 		N_("Host and port-number (host:port) of the HTTPS CONNECT "
 		"proxy, or blank. If it's blank, HTTPS_PROXY environment "
 		"variable is checked as well.")),
+
 	NULL_OPTION_INFO,
 };
 
@@ -248,7 +263,8 @@ struct module http_protocol_module = struct_module(
 	/* submodules: */	NULL,
 	/* data: */		NULL,
 	/* init: */		NULL,
-	/* done: */		done_http
+	/* done: */		done_http,
+	/* getname: */	NULL
 );
 
 
@@ -799,7 +815,7 @@ http_send_header(struct socket *socket)
 
 				proxy_data = straconcat(user, ":", passwd, (char *) NULL);
 				if (proxy_data) {
-					char *proxy_64 = base64_encode(proxy_data);
+					char *proxy_64 = (char*)base64_encode((unsigned char*)proxy_data);
 
 					if (proxy_64) {
 						add_to_string(&header, "Proxy-Authorization: Basic ");
@@ -1017,7 +1033,7 @@ http_send_header(struct socket *socket)
 			id = straconcat(entry->user, ":", entry->password,
 					(char *) NULL);
 			if (id) {
-				char *base64 = base64_encode(id);
+				char *base64 = (char*)base64_encode((unsigned char*)id);
 
 				mem_free_set(&id, base64);
 			}
@@ -1677,7 +1693,7 @@ again:
 	/* XXX: Is there some reason why NOT to follow the Location header
 	 * for any status? If the server didn't mean it, it wouldn't send
 	 * it, after all...? --pasky */
-	if (h == 201 || h == 301 || h == 302 || h == 303 || h == 307) {
+	if (h == 201 || h == 301 || h == 302 || h == 303 || h == 307 || h == 308) {
 		d = parse_header(conn->cached->head, "Location", NULL);
 		if (d) {
 			int use_get_method = (h == 303);
