@@ -29,6 +29,9 @@
 #ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
+#ifdef HAVE_WS2TCPIP_H
+#include <ws2tcpip.h> /* NI_MAXHOST for MinGW */
+#endif
 
 #include "elinks.h"
 
