@@ -11,6 +11,14 @@
 #include "scripting/ruby/core.h"
 #include "scripting/ruby/hooks.h"
 
+static const char *
+get_name_ruby(struct module *xxx)
+{
+	static char elrubyversion[32];
+	snprintf(elrubyversion, 31, "Ruby %s", ruby_version);
+
+	return elrubyversion;
+}
 
 struct module ruby_scripting_module = struct_module(
 	/* name: */		N_("Ruby"),
@@ -19,5 +27,6 @@ struct module ruby_scripting_module = struct_module(
 	/* submodules: */	NULL,
 	/* data: */		NULL,
 	/* init: */		init_ruby,
-	/* done: */		NULL
+	/* done: */		NULL,
+	/* getname: */	get_name_ruby
 );
