@@ -1,7 +1,7 @@
 #ifndef EL__SCRIPTING_SMJS_GLOBAL_OBJECT_H
 #define EL__SCRIPTING_SMJS_GLOBAL_OBJECT_H
 
-#include "ecmascript/spidermonkey-shared.h"
+#include "js/spidermonkey-shared.h"
 
 /* The root of the object hierarchy. If object 'foo' has this as its parent,
  * you can use foo's method and properties with 'foo.<method|property>'. */
