@@ -271,11 +271,6 @@ struct screen_driver_opt {
 	unsigned int utf8_cp:1;
 #endif /* CONFIG_UTF8 */
 
-#ifdef CONFIG_COMBINE
-	/* Whether the terminal supports combining characters. */
-	unsigned int combine:1;
-#endif /* CONFIG_COMBINE */
-
 #ifdef CONFIG_TERMINFO
 	/* Whether use terminfo. */
 	unsigned int terminfo:1;
@@ -321,9 +316,6 @@ static const struct screen_driver_opt dumb_screen_driver_opt = {
 #ifdef CONFIG_UTF8
 	/* utf8_cp: */		0,
 #endif /* CONFIG_UTF8 */
-#ifdef CONFIG_COMBINE
-	/* combine */		0,
-#endif /* CONFIG_COMBINE */
 #ifdef CONFIG_TERMINFO
 	/* terminfo */		0,
 #endif
@@ -348,9 +340,6 @@ static const struct screen_driver_opt vt100_screen_driver_opt = {
 #ifdef CONFIG_UTF8
 	/* utf8_cp: */		0,
 #endif /* CONFIG_UTF8 */
-#ifdef CONFIG_COMBINE
-	/* combine */		0,
-#endif /* CONFIG_COMBINE */
 #ifdef CONFIG_TERMINFO
 	/* terminfo */		0,
 #endif
@@ -375,9 +364,6 @@ static const struct screen_driver_opt linux_screen_driver_opt = {
 #ifdef CONFIG_UTF8
 	/* utf8_cp: */		0,
 #endif /* CONFIG_UTF8 */
-#ifdef CONFIG_COMBINE
-	/* combine */		0,
-#endif /* CONFIG_COMBINE */
 #ifdef CONFIG_TERMINFO
 	/* terminfo */		0,
 #endif
@@ -402,9 +388,6 @@ static const struct screen_driver_opt koi8_screen_driver_opt = {
 #ifdef CONFIG_UTF8
 	/* utf8_cp: */		0,
 #endif /* CONFIG_UTF8 */
-#ifdef CONFIG_COMBINE
-	/* combine */		0,
-#endif /* CONFIG_COMBINE */
 #ifdef CONFIG_TERMINFO
 	/* terminfo */		0,
 #endif
@@ -429,9 +412,6 @@ static const struct screen_driver_opt freebsd_screen_driver_opt = {
 #ifdef CONFIG_UTF8
 	/* utf8_cp: */		0,
 #endif /* CONFIG_UTF8 */
-#ifdef CONFIG_COMBINE
-	/* combine */		0,
-#endif /* CONFIG_COMBINE */
 #ifdef CONFIG_TERMINFO
 	/* terminfo */		0,
 #endif
@@ -456,9 +436,6 @@ static const struct screen_driver_opt fbterm_screen_driver_opt = {
 #ifdef CONFIG_UTF8
 	/* utf8_cp: */		0,
 #endif /* CONFIG_UTF8 */
-#ifdef CONFIG_COMBINE
-	/* combine */		0,
-#endif /* CONFIG_COMBINE */
 #ifdef CONFIG_TERMINFO
 	/* terminfo */		0,
 #endif
@@ -510,10 +487,6 @@ set_screen_driver_opt(struct screen_driver *driver, struct option *term_spec)
 	 * function need not carefully restore options one by one.  */
 	copy_struct(&driver->opt, screen_driver_opts[driver->type]);
 
-#ifdef CONFIG_COMBINE
-	driver->opt.combine = get_opt_bool_tree(term_spec, "combine", NULL);
-#endif /* CONFIG_COMBINE */
-
 #ifdef CONFIG_LIBSIXEL
 	driver->opt.sixel = get_opt_bool_tree(term_spec, "sixel", NULL);
 #endif /* CONFIG_LIBSIXEL */
@@ -858,23 +831,7 @@ add_char_data(struct string *screen, struct screen_driver *driver,
 		}
 		if (data == UCS_NO_CHAR)
 			return;
-#ifdef CONFIG_COMBINE
-		if (data >= UCS_BEGIN_COMBINED && data <= last_combined) {
-			unicode_val_T *text = combined[data - UCS_BEGIN_COMBINED];
 
-			if (driver->opt.combine) {
-				/* XTerm */
-				while (*text != UCS_END_COMBINED) {
-					add_to_string(screen, encode_utf8(*text));
-					text++;
-				}
-				return;
-			} else {
-				/* Others */
-				data = *text;
-			}
-		}
-#endif /* CONFIG_COMBINE */
 		if (!isscreensafe_ucs(data))
 			data = UCS_SPACE;
 		add_to_string(screen, encode_utf8(data));
@@ -1466,18 +1423,6 @@ add_char_true(struct string *screen, struct screen_driver *driver,
 		clear_bitfield_bit(screen->dirty, y);				\
 										\
 		for (; x <= xmax; x++, current++, pos++) {			\
-			/*  Workaround for terminals without
-			 *  "eat_newline_glitch (xn)", e.g., the cons25 family
-			 *  of terminals and cygwin terminal.
-			 *  It prevents display distortion, but char at bottom
-			 *  right of terminal will not be drawn.
-			 *  A better fix would be to correctly detects
-			 *  terminal type, and/or add a terminal option for
-			 *  this purpose. */					\
-										\
-			if (is_last_line && x == xmax)				\
-				break;						\
-										\
 			if (compare_bg_color(pos->c.color, current->c.color)) {	\
 				/* No update for exact match. */		\
 				if (compare_fg_color(pos->c.color, current->c.color)\
@@ -1499,6 +1444,18 @@ add_char_true(struct string *screen, struct screen_driver *driver,
 			continue;						\
 		}								\
 		add_cursor_move_to_string(image_, y + 1, 1);			\
+																\
+		/*  Workaround for terminals without
+		 *  "eat_newline_glitch (xn)", e.g., the cons25 family
+		 *  of terminals and cygwin terminal.
+		 *  It prevents display distortion, but char at bottom
+		 *  right of terminal will not be drawn.
+		 *  A better fix would be to correctly detects
+		 *  terminal type, and/or add a terminal option for
+		 *  this purpose. */					\
+		if (is_last_line) {	\
+			xmax--;	\
+		}	\
 		for (pos = start_of_line, x = 0; x <= xmax; x++, pos++) {	\
 			ADD_CHAR(image_, driver_, pos, state_);	\
 		}							\
@@ -1699,5 +1656,6 @@ struct module terminal_screen_module = struct_module(
 	/* submodules: */	NULL,
 	/* data: */		NULL,
 	/* init: */		NULL,
-	/* done: */		done_screen_drivers
+	/* done: */		done_screen_drivers,
+	/* getname: */	NULL
 );
