@@ -67,6 +67,7 @@ clean_temporary_files(void)
 	foreach_hash_item (item, *temporary_files, i) {
 		if (item->key) {
 			unlink(item->key);
+			mem_free_set(&item->key, NULL);
 		}
 	}
 	free_hash(&temporary_files);
@@ -285,6 +286,11 @@ destroy_terminal(struct terminal *term)
 	}
 
 	object_unlock(term->spec);
+
+	if (term->closed_tab_uri) {
+		done_uri(term->closed_tab_uri);
+	}
+
 	mem_free(term);
 	check_if_no_terminal();
 }
@@ -306,12 +312,11 @@ check_if_no_terminal(void)
 void
 exec_thread(char *path, int p)
 {
-	int plen = strlen(path + 1) + 2;
-
 #if defined(HAVE_SETPGID) && !defined(CONFIG_OS_BEOS) && !defined(HAVE_BEGINTHREAD)
 	if (path[0] == TERM_EXEC_NEWWIN) setpgid(0, 0);
 #endif
 #ifndef WIN32
+	int plen = strlen(path + 1) + 2;
 	if (path[0] == TERM_EXEC_BG)
 		exe_no_stdin(path + 1);
 	else
@@ -595,5 +600,6 @@ struct module terminal_module = struct_module(
 	/* submodules: */	terminal_submodules,
 	/* data: */		NULL,
 	/* init: */		NULL,
-	/* done: */		NULL
+	/* done: */		NULL,
+	/* getname: */	NULL
 );
