@@ -2,6 +2,7 @@
 #define EL__TERMINAL_TERMINAL_H
 
 #include "config/options.h"
+#include "protocol/uri.h"
 #include "terminal/event.h"
 #include "util/lists.h"
 
@@ -179,9 +180,11 @@ struct terminal {
 
 #ifdef CONFIG_LIBSIXEL
 	LIST_OF(struct image) images;
+#endif
 	int cell_width;
 	int cell_height;
-#endif
+
+	struct uri *closed_tab_uri;
 };
 
 #define do_not_ignore_next_mouse_event(term) \
