@@ -31,8 +31,7 @@ terminfo_clear_screen(void)
 {
 	char *res = tiparm(clear_screen);
 
-	if (res) return res;
-	return "";
+	return res ?: "";
 }
 
 const char *
@@ -40,8 +39,7 @@ terminfo_set_bold(int arg)
 {
 	char *res = tiparm(arg ? enter_bold_mode : exit_attribute_mode);
 
-	if (res) return res;
-	return "";
+	return res ?: "";
 }
 
 const char *
@@ -49,8 +47,7 @@ terminfo_set_italics(int arg)
 {
 	char *res = tiparm(arg ? enter_italics_mode : exit_italics_mode);
 
-	if (res) return res;
-	return "";
+	return res ?: "";
 }
 
 const char *
@@ -58,8 +55,15 @@ terminfo_set_underline(int arg)
 {
 	char *res = tiparm(arg ? enter_underline_mode : exit_underline_mode);
 
-	if (res) return res;
-	return "";
+	return res ?: "";
+}
+
+const char *
+terminfo_set_strike(int arg)
+{
+	char *res = tigetstr(arg ? "smxx" : "rmxx");
+
+	return res ?: "";
 }
 
 const char *
@@ -67,8 +71,7 @@ terminfo_set_background(int arg)
 {
 	char *res = tiparm(set_a_background, arg);
 
-	if (res) return res;
-	return "";
+	return res ?: "";
 }
 
 const char *
@@ -76,8 +79,15 @@ terminfo_set_foreground(int arg)
 {
 	char *res = tiparm(set_a_foreground, arg);
 
-	if (res) return res;
-	return "";
+	return res ?: "";
+}
+
+const char *
+terminfo_set_standout(int arg)
+{
+	char *res = tiparm(arg ? enter_standout_mode : exit_standout_mode);
+
+	return res ?: "";
 }
 
 int
@@ -91,6 +101,5 @@ terminfo_cursor_address(int y, int x)
 {
 	char *res = tiparm(cursor_address, y, x);
 
-	if (res) return res;
-	return "";
+	return res ?: "";
 }
