@@ -28,6 +28,7 @@
 #include "intl/libintl.h"
 #include "main/select.h"
 #include "main/main.h"
+#include "mime/mime.h"
 #include "network/connection.h"
 #include "network/state.h"
 #include "osdep/ascii.h"
@@ -104,6 +105,8 @@ static const unsigned char frame_simplify[FRAME_CHARS_END - FRAME_CHARS_BEGIN]
 	0xC5, '+' , '+' , '#' , 0xDB, 0xDB, 0xDB, 0xDB  /* 0xD8...0xDF */
 };
 
+static void dump_print(const char *option, const char *url);
+
 /** Initialize dump_output::frame for the specified codepage.
  *
  * If the codepage does not support all the box-drawing characters
@@ -502,11 +505,15 @@ dump_formatted(int fd, struct download *download, struct cache_entry *cached)
 	o.dump = 1;
 
 #ifdef CONFIG_SCRIPTING
-	maybe_pre_format_html(cached, NULL);
+	if (get_opt_bool("document.dump.scripting_enabled", NULL)) {
+		maybe_pre_format_html(cached, NULL);
+	}
 #endif
 
 	init_vs(&vs, cached->uri, -1);
 
+	(void)get_content_type(cached);
+
 	render_document(&vs, &formatted, &o);
 
 	out = dump_output_alloc(fd, NULL, o.cp);
@@ -545,9 +552,13 @@ dump_formatted(int fd, struct download *download, struct cache_entry *cached)
 #endif
 		}
 
-		if (!error)
+		if (!error) {
 			dump_references(formatted.document, fd, out->buf);
+		}
+		char *url = get_uri_string(cached->uri, URI_PUBLIC);
 
+		dump_print("document.dump.footer", url);
+		mem_free_if(url);
 		mem_free(out);
 	} /* if out */
 
@@ -600,7 +611,7 @@ nextfrag:
 }
 
 static char *
-subst_url(char *str, struct string *url)
+subst_url(char *str, const char *url)
 {
 	struct string string;
 
@@ -644,7 +655,7 @@ subst_url(char *str, struct string *url)
 		str++;
 		switch (*str) {
 			case 'u':
-				if (url) add_string_to_string(&string, url);
+				if (url) add_to_string(&string, url);
 				break;
 		}
 
@@ -655,7 +666,7 @@ subst_url(char *str, struct string *url)
 }
 
 static void
-dump_print(const char *option, struct string *url)
+dump_print(const char *option, const char *url)
 {
 	char *str = get_opt_str(option, NULL);
 
@@ -772,12 +783,8 @@ dump_next(LIST_OF(struct string_list_item) *url_list)
 			first = 0;
 		}
 
-		dump_print("document.dump.header", &item->string);
+		dump_print("document.dump.header", item->string.source);
 		dump_start(item->string.source);
-		/* XXX: I think it ought to print footer at the end of
-		 * the whole dump (not only this file). Testing required.
-		 * --pasky */
-		dump_print("document.dump.footer", &item->string);
 
 	} else {
 		free_string_list(&done_list);
