@@ -9,6 +9,12 @@
 
 #include "elinks.h"
 
+#ifdef CONFIG_LIBDOM
+#include "js/libdom/dom.h"
+#include <dom/dom.h>
+#include <dom/bindings/hubbub/parser.h>
+#endif
+
 #include "bfu/listmenu.h"
 #include "bfu/menu.h"
 #include "bfu/style.h"
@@ -16,11 +22,16 @@
 #include "dialogs/status.h"
 #include "document/document.h"
 #include "document/forms.h"
+#include "document/html/iframes.h"
 #include "document/html/renderer.h"
 #include "document/options.h"
 #include "document/view.h"
-#include "ecmascript/ecmascript.h"
-#include "intl/gettext/libintl.h"
+
+#if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
+#include "js/ecmascript-c.h"
+#endif
+
+#include "intl/libintl.h"
 #include "main/object.h"
 #include "protocol/uri.h"
 #include "session/session.h"
@@ -49,36 +60,11 @@
  * --pasky */
 
 
-static int
+int
 current_link_evhook(struct document_view *doc_view, enum script_event_hook_type type)
 {
-#ifdef CONFIG_ECMASCRIPT
-	struct link *link;
-	struct script_event_hook *evhook;
-
-	assert(doc_view && doc_view->vs);
-	link = get_current_link(doc_view);
-	if (!link) return -1;
-	if (!link->event_hooks) return -1;
-
-	if (!doc_view->vs->ecmascript) return -1;
-
-	foreach (evhook, *link->event_hooks) {
-		unsigned char *ret;
-
-		if (evhook->type != type) continue;
-		ret = evhook->src;
-		while ((ret = strstr((const char *)ret, "return ")))
-			while (*ret != ' ') *ret++ = ' ';
-		{
-			struct string src = INIT_STRING(evhook->src, strlen(evhook->src));
-			/* TODO: Some even handlers return a bool. */
-			if (!ecmascript_eval_boolback(doc_view->vs->ecmascript, &src))
-				return 0;
-		}
-	}
-
-	return 1;
+#if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
+	return ecmascript_current_link_evhook(doc_view, type);
 #else
 	return -1;
 #endif
@@ -96,7 +82,6 @@ do { \
 	current_link_evhook(dv, SEVHOOK_ONBLUR); \
 } while (0)
 
-
 void
 set_link(struct document_view *doc_view)
 {
@@ -111,7 +96,7 @@ set_link(struct document_view *doc_view)
 static inline int
 get_link_cursor_offset(struct document_view *doc_view, struct link *link)
 {
-	struct form_control *fc;
+	struct el_form_control *fc;
 	struct form_state *fs;
 #ifdef CONFIG_UTF8
 	/* The encoding of form fields depends on the terminal,
@@ -123,6 +108,7 @@ get_link_cursor_offset(struct document_view *doc_view, struct link *link)
 		case LINK_CHECKBOX:
 			return 1;
 
+		case LINK_SELECT:
 		case LINK_BUTTON:
 			return 2;
 
@@ -133,8 +119,8 @@ get_link_cursor_offset(struct document_view *doc_view, struct link *link)
 				return 0;
 #ifdef CONFIG_UTF8
 			else if (utf8) {
-				unsigned char *scroll = fs->value + fs->vpos;
-				unsigned char *point = fs->value + fs->state;
+				char *scroll = fs->value + fs->vpos;
+				char *point = fs->value + fs->state;
 
 				if (fs->type == FC_PASSWORD)
 					return utf8_ptr2chars(scroll, point);
@@ -156,7 +142,6 @@ get_link_cursor_offset(struct document_view *doc_view, struct link *link)
 
 		case LINK_HYPERTEXT:
 		case LINK_MAP:
-		case LINK_SELECT:
 			return 0;
 	}
 
@@ -166,12 +151,12 @@ get_link_cursor_offset(struct document_view *doc_view, struct link *link)
 /** Initialise a static template character with the colour and attributes
  * appropriate for an active link and return that character. */
 static inline struct screen_char *
-init_link_drawing(struct document_view *doc_view, struct link *link, int invert)
+init_link_drawing(struct document_view *doc_view, struct link *link, int invert, int input)
 {
 	struct document_options *doc_opts;
 	static struct screen_char template_;
-	enum color_flags color_flags;
-	enum color_mode color_mode;
+	color_flags_T color_flags;
+	color_mode_T color_mode;
 	struct color_pair colors;
 
 	template_.attr = SCREEN_ATTR_STANDOUT;
@@ -181,15 +166,22 @@ init_link_drawing(struct document_view *doc_view, struct link *link, int invert)
 	color_flags = (doc_opts->color_flags | COLOR_DECREASE_LIGHTNESS);
 	color_mode = doc_opts->color_mode;
 
-	if (doc_opts->active_link.underline)
+	/* avoid underlining text input areas which has underscores */
+	if (doc_opts->active_link.underline && link->type != LINK_FIELD)
 		template_.attr |= SCREEN_ATTR_UNDERLINE;
 
-	if (doc_opts->active_link.bold)
+	/* keep forms in bold */
+	if (doc_opts->active_link.bold || link->type != LINK_HYPERTEXT)
 		template_.attr |= SCREEN_ATTR_BOLD;
 
 	if (doc_opts->active_link.enable_color) {
-		colors.foreground = doc_opts->active_link.color.foreground;
-		colors.background = doc_opts->active_link.color.background;
+		if (input) {
+			colors.foreground = doc_opts->active_link.insert_mode_color.foreground;
+			colors.background = doc_opts->active_link.insert_mode_color.background;
+		} else {
+			colors.foreground = doc_opts->active_link.color.foreground;
+			colors.background = doc_opts->active_link.color.background;
+		}
 	} else {
 		colors.foreground = link->color.foreground;
 		colors.background = link->color.background;
@@ -232,6 +224,7 @@ draw_current_link(struct session *ses, struct document_view *doc_view)
 	struct link *link;
 	int cursor_offset;
 	int xpos, ypos;
+	int invert, input;
 	int i;
 
 	assert(term && doc_view && doc_view->vs);
@@ -243,8 +236,9 @@ draw_current_link(struct session *ses, struct document_view *doc_view)
 	link = get_current_link(doc_view);
 	if (!link) return;
 
-	i = !link_is_textinput(link) || ses->insert_mode == INSERT_MODE_OFF;
-	template_ = init_link_drawing(doc_view, link, i);
+	invert = !link_is_textinput(link) || ses->insert_mode == INSERT_MODE_OFF;
+	input = ses->insert_mode == INSERT_MODE_ON;
+	template_ = init_link_drawing(doc_view, link, invert, input);
 	if (!template_) return;
 
 	xpos = doc_view->box.x - doc_view->vs->x;
@@ -272,7 +266,7 @@ draw_current_link(struct session *ses, struct document_view *doc_view)
 
 		if (i == cursor_offset) {
 			int blockable = (!link_is_textinput(link)
-					 && co->c.color != template_->c.color);
+					 && memcmp(co->c.color, template_->c.color, sizeof(template_->c.color)));
 
 			set_cursor(term, x, y, blockable);
 			set_window_ptr(ses->tab, x, y);
@@ -302,7 +296,7 @@ draw_link(struct terminal *term, struct document_view *doc_view,
 			struct screen_char *ch;
 
 			ch = get_char(term, x + xpos, y + ypos);
-			copy_struct(ch, &doc_view->document->data[y].chars[x]);
+			copy_struct(ch, &doc_view->document->data[y].ch.chars[x]);
 			set_screen_dirty(term->screen, y + ypos, y + ypos);
 		}
 	}
@@ -347,8 +341,9 @@ highlight_links_with_prefixes_that_start_with_n(struct terminal *term,
 				int x = link->points[i].x + xoffset;
 				int y = link->points[i].y + yoffset;
 
-				if (is_in_box(&doc_view->box, x, y))
+				if (is_in_box(&doc_view->box, x, y)) {
 					draw_char_color(term, x, y, color);
+				}
 			}
 		}
 	}
@@ -510,6 +505,62 @@ next_link_in_view_(struct document_view *doc_view, int current, int direction,
 	document = doc_view->document;
 	vs = doc_view->vs;
 
+	if (document_has_iframes(document)) {
+		struct document_view *dw;
+
+		if (direction > 0) {
+			int j = 0;
+
+			foreach (dw, doc_view->session->scrn_iframes) {
+				if (vs->current_link != document->iframe_desc->iframe_desc[j].nlink - 1) {
+					goto next;
+				}
+				if (dw->document->nlinks > 0) {
+					vs = dw->vs;
+					current_link_blur(doc_view);
+
+					if (fn(dw, &dw->document->links[0])) {
+						dw->parent_doc_view = dw->session->doc_view;
+						dw->session->doc_view = dw;
+						vs->current_link = 0;
+
+						if (cntr) cntr(dw, &dw->document->links[0]);
+						current_link_hover(dw);
+
+						return 1;
+					}
+				}
+next:
+				j++;
+			}
+		} else {
+			int j = document->iframe_desc->n - 1;
+
+			foreachback (dw, doc_view->session->scrn_iframes) {
+				if (vs->current_link != document->iframe_desc->iframe_desc[j].nlink) {
+					goto nextback;
+				}
+				if (dw->document->nlinks > 0) {
+					int last = dw->document->nlinks - 1;
+					vs = dw->vs;
+					current_link_blur(doc_view);
+
+					if (fn(dw, &dw->document->links[last])) {
+						dw->parent_doc_view = dw->session->doc_view;
+						dw->session->doc_view = dw;
+						vs->current_link = last;
+						if (cntr) cntr(dw, &dw->document->links[last]);
+						current_link_hover(dw);
+
+						return 1;
+					}
+				}
+nextback:
+				j--;
+			}
+		}
+	}
+
 	get_visible_links_range(doc_view, &start, &end);
 
 	current_link_blur(doc_view);
@@ -519,6 +570,7 @@ next_link_in_view_(struct document_view *doc_view, int current, int direction,
 	while (current >= start && current <= end) {
 		if (fn(doc_view, &document->links[current])) {
 			vs->current_link = current;
+
 			if (cntr) cntr(doc_view, &document->links[current]);
 			current_link_hover(doc_view);
 			return 1;
@@ -526,7 +578,38 @@ next_link_in_view_(struct document_view *doc_view, int current, int direction,
 		current += direction;
 	}
 
+	if (doc_view->parent_doc_view &&
+		((current <= 0 && direction < 0) || (current == document->nlinks && direction > 0))) {
+		int j = 0;
+		struct document_view *dw = NULL;
+		struct session *ses = doc_view->parent_doc_view->session;
+
+		foreach (dw, ses->scrn_iframes) {
+			if (dw == doc_view) {
+				break;
+			}
+			j++;
+		}
+		document = doc_view->parent_doc_view->document;
+		vs = doc_view->parent_doc_view->vs;
+		if (direction < 0) {
+			current = document->iframe_desc->iframe_desc[j].nlink - 1;
+		} else {
+			current = document->iframe_desc->iframe_desc[j].nlink;
+		}
+
+		if (fn(doc_view->parent_doc_view, &document->links[current])) {
+			vs->current_link = current;
+			doc_view = doc_view->parent_doc_view;
+			doc_view->session->doc_view = doc_view;
+
+			if (cntr) cntr(doc_view, &document->links[current]);
+			current_link_hover(doc_view);
+			return 1;
+		}
+	}
 	vs->current_link = -1;
+
 	return 0;
 }
 
@@ -723,6 +806,7 @@ next_link_in_dir(struct document_view *doc_view, int dir_x, int dir_y)
 
 	current_link_blur(doc_view);
 	vs->current_link = -1;
+
 	return 0;
 
 chose_link:
@@ -731,6 +815,7 @@ chose_link:
 	vs->current_link = get_link_index(document, link);
 	set_pos_x(doc_view, link);
 	current_link_hover(doc_view);
+
 	return 1;
 }
 
@@ -842,6 +927,7 @@ find_link(struct document_view *doc_view, int direction, int page_mode)
 	doc_view->vs->current_link = link_pos;
 	set_pos_x(doc_view, link);
 	current_link_hover(doc_view);
+
 	return;
 
 nolink:
@@ -883,8 +969,8 @@ get_link_uri(struct session *ses, struct document_view *doc_view,
 	switch (link->type) {
 		case LINK_HYPERTEXT:
 		case LINK_MAP:
-			if (link->where) return get_uri(link->where, 0);
-			return get_uri(link->where_img, 0);
+			if (link->where) return get_uri(link->where, URI_NONE);
+			return get_uri(link->where_img, URI_NONE);
 
 		case LINK_BUTTON:
 		case LINK_FIELD:
@@ -898,15 +984,15 @@ get_link_uri(struct session *ses, struct document_view *doc_view,
 
 static int
 call_onsubmit_and_submit(struct session *ses, struct document_view *doc_view,
-			 struct form_control *fc, int do_reload)
+			 struct el_form_control *fc, int do_reload)
 {
 	struct uri *uri = NULL;
-	enum cache_mode mode = do_reload ? CACHE_MODE_FORCE_RELOAD : CACHE_MODE_NORMAL;
+	cache_mode_T mode = do_reload ? CACHE_MODE_FORCE_RELOAD : CACHE_MODE_NORMAL;
 
 	assert(fc->form); /* regardless of whether there is a FORM element */
 	if_assert_failed return 0;
 
-#ifdef CONFIG_ECMASCRIPT
+#if  defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
 	/* If the form has multiple submit buttons, this does not
 	 * explicitly tell the ECMAScript code which of them was
 	 * pressed.  W3C DOM Level 3 doesn't seem to include such a
@@ -919,8 +1005,9 @@ call_onsubmit_and_submit(struct session *ses, struct document_view *doc_view,
 			struct ecmascript_interpreter *interpreter;
 			int res;
 
-			if (vs->ecmascript_fragile)
+			if (vs->ecmascript_fragile) {
 				ecmascript_reset_state(vs);
+			}
 			interpreter = vs->ecmascript;
 			/* If there is an onsubmit script and we want
 			 * to run it, but the ECMAScript interpreter
@@ -943,7 +1030,7 @@ call_onsubmit_and_submit(struct session *ses, struct document_view *doc_view,
 			if (!res) return 1;
 		}
 	}
-#endif	/* CONFIG_ECMASCRIPT */
+#endif	/* CONFIG_ECMASCRIPT_SMJS */
 
 	uri = get_form_uri(ses, doc_view, fc);
 	if (!uri) return 0;
@@ -960,8 +1047,14 @@ goto_link(struct session *ses, struct document_view *doc_view, struct link *link
 	assert(link && doc_view && ses);
 	if_assert_failed return NULL;
 
+	if (doc_view->vs) {
+		doc_view->prev_y = doc_view->vs->y;
+	} else {
+		doc_view->prev_y = 0;
+	}
+
 	if (link_is_form(link)) {
-		struct form_control *fc = link->data.form_control;
+		struct el_form_control *fc = link->data.form_control;
 
 		if (fc->type != FC_BUTTON
 		    && !call_onsubmit_and_submit(ses, doc_view, fc, do_reload))
@@ -978,7 +1071,7 @@ goto_link(struct session *ses, struct document_view *doc_view, struct link *link
 		goto_imgmap(ses, uri, link->target);
 
 	} else {
-		enum cache_mode mode = do_reload ? CACHE_MODE_FORCE_RELOAD
+		cache_mode_T mode = do_reload ? CACHE_MODE_FORCE_RELOAD
 						 : CACHE_MODE_NORMAL;
 
 		goto_uri_frame(ses, uri, link->target, mode);
@@ -1006,7 +1099,7 @@ static enum frame_event_status
 activate_link(struct session *ses, struct document_view *doc_view,
               struct link *link, int do_reload)
 {
-	struct form_control *link_fc;
+	struct el_form_control *link_fc;
 	struct form_state *fs;
 	struct form *form;
 
@@ -1044,7 +1137,7 @@ activate_link(struct session *ses, struct document_view *doc_view,
 		 * @fs in the loop. */
 		fs->state = 1;
 		foreach (form, doc_view->document->forms) {
-			struct form_control *fc;
+			struct el_form_control *fc;
 
 			if (form != link_fc->form)
 				continue;
@@ -1170,6 +1263,7 @@ jump_to_link_number(struct session *ses, struct document_view *doc_view, int n)
 	if (n < 0 || n >= doc_view->document->nlinks) return;
 	current_link_blur(doc_view);
 	doc_view->vs->current_link = n;
+
 	if (ses->navigate_mode == NAVIGATE_CURSOR_ROUTING) {
 		struct link *link = get_current_link(doc_view);
 		int offset = get_link_cursor_offset(doc_view, link);
@@ -1205,7 +1299,7 @@ goto_link_number_do(struct session *ses, struct document_view *doc_view, int n)
 }
 
 void
-goto_link_number(struct session *ses, unsigned char *num)
+goto_link_number(struct session *ses, char *num)
 {
 	struct document_view *doc_view;
 
@@ -1218,7 +1312,7 @@ goto_link_number(struct session *ses, unsigned char *num)
 }
 
 void
-goto_link_symbol(struct session *ses, unsigned char *sym)
+goto_link_symbol(struct session *ses, char *sym)
 {
 	char *symkey = get_opt_str("document.browse.links.label_key", ses);
 	struct document_view *doc_view;
@@ -1294,11 +1388,11 @@ try_document_key(struct session *ses, struct document_view *doc_view,
 void
 link_menu(struct terminal *term, void *xxx, void *ses_)
 {
-	struct session *ses = ses_;
+	struct session *ses = (struct session *)ses_;
 	struct document_view *doc_view;
 	struct link *link;
 	struct menu_item *mi;
-	struct form_control *fc;
+	struct el_form_control *fc;
 
 	assert(term && ses);
 	if_assert_failed return;
@@ -1428,7 +1522,7 @@ end:
 }
 
 /** Return current link's title. */
-unsigned char *
+char *
 get_current_link_title(struct document_view *doc_view)
 {
 	struct link *link;
@@ -1442,7 +1536,7 @@ get_current_link_title(struct document_view *doc_view)
 	link = get_current_link(doc_view);
 
 	if (link && link->title && *link->title) {
-		unsigned char *link_title, *src;
+		char *link_title, *src;
 		struct conv_table *convert_table;
 
 		convert_table = get_translation_table(doc_view->document->cp,
@@ -1459,7 +1553,7 @@ get_current_link_title(struct document_view *doc_view)
 		if (link_title && !doc_view->document->options.utf8)
 #endif /* CONFIG_UTF8 */
 			for (src = link_title; *src; src++)
-				if (!isprint(*src) || iscntrl(*src))
+				if (iscntrl((unsigned char)*src))
 					*src = '*';
 
 		return link_title;
@@ -1468,7 +1562,7 @@ get_current_link_title(struct document_view *doc_view)
 	return NULL;
 }
 
-unsigned char *
+char *
 get_current_link_info(struct session *ses, struct document_view *doc_view)
 {
 	struct link *link;
@@ -1487,7 +1581,7 @@ get_current_link_info(struct session *ses, struct document_view *doc_view)
 	if (!link_is_form(link)) {
 		struct terminal *term = ses->tab->term;
 		struct string str;
-		unsigned char *uristring = link->where;
+		char *uristring = link->where;
 
 		if (!init_string(&str)) return NULL;
 
